/*
                              AUX Pipes

	Stat codes used in identifying the data segements in
	AUX pipes.

	AUX Pipes are used by ShipWars server (swserv) to indicate
	its status (ie uptime, connections, objects, etc) by FIFO
	pipes. AUX Pipes are in actuality, FIFO pipes.


 */

#ifndef AUXSTATCODES_H
#define AUXSTATCODES_H


/*
 *	Maximum data allowed in an AUX Pipe:
 *
 *	May need to adjust this value, check limit given by os!!
 */
#define AUXPIPE_MAX_LEN		4096


/*
 *	Maximum length of stat string prefix:
 */
#define STAT_PFX_MAX_LEN	128


/*
 *   Stat String Prefixes:
 *
 *	These are prefix values for stat strings sent through
 *	the pipes.
 *
 *	Example:
 *	    "%mem 1534 124 213 34"
 *
 *	Would indicate memory usage stats for various things
 *	that the server has loaded. (This is JUST AN EXAMPLE.)
 *
 *	Each stat string will follow with a number of arguments
 *	and then a \n character denoting the end.
 *
 *	I don't have a list of arguments for each stat string, however
 *	you can look at the sprintf()'s in the server's auxpipe.c source.
 */

#define STAT_PFX_NUMCON		"%numcon"
#define STAT_PFX_UNVTITLE	"%unvtitle"
#define STAT_PFX_EXEC		"%exec"
#define STAT_PFX_MEM		"%mem"
#define STAT_PFX_NSAVE		"%nsave"
#define STAT_PFX_NEXPORT	"%nexport"
#define STAT_PFX_NUMOBJ		"%numobj"
#define STAT_PFX_UPTIME		"%uptime"


#endif /* AUXSTATCODES_H */
