#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#ifdef __linux__
# include <linux/soundcard.h>
#endif

#include "mixer.h"




int GET_MIXER_NUM_BY_NAME(int fd, char *name);


int MixerSetBass(int fd, double value);
int MixerSetCD(int fd, double value1, double value2);
int MixerSetGainIn(int fd, double value1, double value2);
int MixerSetGainOut(int fd, double value1, double value2);
int MixerSetLine(int fd, double value);
int MixerSetLine1(int fd, double value);
int MixerSetLine2(int fd, double value);
int MixerSetLine3(int fd, double value);
int MixerSetMic(int fd, double value1, double value2);
int MixerSetMix(int fd, double value);
int MixerSetPCM(int fd, double value1, double value2);
int MixerSetPCM2(int fd, double value1, double value2);
int MixerSetRec(int fd, double value);
int MixerSetSpeaker(int fd, double value);
int MixerSetSynth(int fd, double value1, double value2);
int MixerSetTreble(int fd, double value);
int MixerSetVolume(int fd, double value1, double value2);


/*
 *	Returns the mixer device's number matching name or
 *	-1 on failed match.
 */
int GET_MIXER_NUM_BY_NAME(int fd, char *name)
{
	int i;
	int mixermask;
        const char *reg_name[] = SOUND_DEVICE_NAMES;


	if((fd < 0) ||
           (name == NULL)
	)
	    return(-1);


        /* Get mixer info mask. */
        if(ioctl(fd, SOUND_MIXER_READ_DEVMASK, &mixermask))
        {
	    /* Could not get mixer info mask. */
            return(-1);
        }

        for(i = 0; i < SOUND_MIXER_NRDEVICES; i++)
        {
            if(((1 << i) & mixermask) &&
               !strcmp(name, reg_name[i])
            )
                break;
        }

        if(i < SOUND_MIXER_NRDEVICES)
	    return(i);
	else
	    return(-1);
} 




int MixerSetBass(int fd, double value)
{
        int i, packed_value;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "bass");
        if(i < 0)
            return(-1);

        if(value > 1) 
            value = 1;
        if(value < 0) 
            value = 0;

        value = value * 100 / 255;
        packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetCD(int fd, double value1, double value2)
{
        int i, packed_value;
	int stereomask;
	double avg;
	int left, right;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "cd");
        if(i < 0)
            return(-1);

        /* Get mixer stereo mask. */
        if(
            ioctl(
                fd,
                SOUND_MIXER_READ_STEREODEVS,
                &stereomask
            )  
        )
            return(-1);


        if(value1 > 1) 
            value1 = 1;
        if(value1 < 0) 
            value1 = 0;

        if(value2 > 1)
            value2 = 1;
        if(value2 < 0)
            value2 = 0;

        value1 = value1 * 100 / 255;
        value2 = value2 * 100 / 255;
          
        /* Sanitize left and right for non-stereo mixer device. */
        if((value1 != value2) && !((1 << i) & stereomask))
        {
            /* Average the two values. */
            avg = (value1 + value2) / 2;
            value1 = avg;
            value2 = avg;
        }
 
        left = (unsigned int)(value1 * (double)255);
        right = (unsigned int)(value2 * (double)255);
        
        packed_value = (right << 8) + left;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetGainIn(int fd, double value1, double value2)
{
        int i, packed_value;
        int stereomask;
        double avg;
        int left, right;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "igain");
        if(i < 0)
            return(-1);

        /* Get mixer stereo mask. */
        if(
            ioctl(
                fd,
                SOUND_MIXER_READ_STEREODEVS,
                &stereomask
            )
        ) 
            return(-1);

        if(value1 > 1)
            value1 = 1;
        if(value1 < 0) 
            value1 = 0;

        if(value2 > 1)
            value2 = 1;
        if(value2 < 0)
            value2 = 0;

        value1 = value1 * 100 / 255;  
        value2 = value2 * 100 / 255;

        /* Sanitize left and right for non-stereo mixer device. */
        if((value1 != value2) && !((1 << i) & stereomask))
        {
            /* Average the two values. */
            avg = (value1 + value2) / 2;
            value1 = avg;
            value2 = avg;
        }

        left = (unsigned int)(value1 * (double)255);
        right = (unsigned int)(value2 * (double)255);

        packed_value = (right << 8) + left;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetGainOut(int fd, double value1, double value2)
{
        int i, packed_value;
        int stereomask;
        double avg;
        int left, right;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "ogain");
        if(i < 0)
            return(-1);

        /* Get mixer stereo mask. */
        if(
            ioctl( 
                fd,
                SOUND_MIXER_READ_STEREODEVS,
                &stereomask
            )
        )
            return(-1);

        if(value1 > 1)   
            value1 = 1;  
        if(value1 < 0)
            value1 = 0;

        if(value2 > 1)
            value2 = 1;
        if(value2 < 0)
            value2 = 0;

        value1 = value1 * 100 / 255;
        value2 = value2 * 100 / 255;

        /* Sanitize left and right for non-stereo mixer device. */
        if((value1 != value2) && !((1 << i) & stereomask))
        {
            /* Average the two values. */
            avg = (value1 + value2) / 2;
            value1 = avg;
            value2 = avg;
        }

        left = (unsigned int)(value1 * (double)255);
        right = (unsigned int)(value2 * (double)255);

        packed_value = (right << 8) + left;

        return(  
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetLine(int fd, double value)
{
        int i, packed_value;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "line");
        if(i < 0)
            return(-1);

        if(value > 1)  
            value = 1;
        if(value < 0)
            value = 0;

        packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetLine1(int fd, double value)
{
        int i, packed_value;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "line1");
        if(i < 0)
            return(-1);

        if(value > 1) 
            value = 1;
        if(value < 0) 
            value = 0;

        packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetLine2(int fd, double value)
{
        int i, packed_value;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "line2");
        if(i < 0)
            return(-1);

        if(value > 1)  
            value = 1;
        if(value < 0) 
            value = 0;

        packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetLine3(int fd, double value)
{
        int i, packed_value;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "line3");
        if(i < 0)
            return(-1);

        if(value > 1)  
            value = 1;
        if(value < 0)
            value = 0;

        packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetMic(int fd, double value1, double value2)
{
        int i, packed_value;
        int stereomask;
        double avg;
        int left, right;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */  
        i = GET_MIXER_NUM_BY_NAME(fd, "mic");
        if(i < 0)
            return(-1);

        /* Get mixer stereo mask. */
        if(
            ioctl(
                fd,
                SOUND_MIXER_READ_STEREODEVS,
                &stereomask
            )
        ) 
            return(-1);

        if(value1 > 1)   
            value1 = 1;  
        if(value1 < 0)
            value1 = 0;
        
        if(value2 > 1)
            value2 = 1;
        if(value2 < 0)
            value2 = 0;

        value1 = value1 * 100 / 255;
        value2 = value2 * 100 / 255;
 
        /* Sanitize left and right for non-stereo mixer device. */
        if((value1 != value2) && !((1 << i) & stereomask))
        {
            /* Average the two values. */
            avg = (value1 + value2) / 2;
            value1 = avg;
            value2 = avg;
        }

        left = (unsigned int)(value1 * (double)255);
        right = (unsigned int)(value2 * (double)255);

        packed_value = (right << 8) + left;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetMix(int fd, double value)
{
        int i, packed_value;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "mix");
        if(i < 0)
            return(-1);

        if(value > 1) 
            value = 1;
        if(value < 0) 
            value = 0;

        packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetPCM(int fd, double value1, double value2)
{
        int i, packed_value;
        int stereomask;
        double avg;
        int left, right;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "pcm");
        if(i < 0)
            return(-1);

        /* Get mixer stereo mask. */
        if(
            ioctl(
                fd,
                SOUND_MIXER_READ_STEREODEVS,
                &stereomask
            )
        )
            return(-1);

        if(value1 > 1)   
            value1 = 1;  
        if(value1 < 0)
            value1 = 0;

        if(value2 > 1)
            value2 = 1;
        if(value2 < 0)
            value2 = 0;

        value1 = value1 * 100 / 255;
        value2 = value2 * 100 / 255;

        /* Sanitize left and right for non-stereo mixer device. */
        if((value1 != value2) && !((1 << i) & stereomask))
        {
            /* Average the two values. */
            avg = (value1 + value2) / 2;
            value1 = avg;
            value2 = avg;
        }

        left = (unsigned int)(value1 * (double)255);
        right = (unsigned int)(value2 * (double)255);

        packed_value = (right << 8) + left;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetPCM2(int fd, double value1, double value2)
{
        int i, packed_value;
        int stereomask;
        double avg;
        int left, right;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "pcm2");
        if(i < 0)
            return(-1);

        /* Get mixer stereo mask. */
        if(
            ioctl(
                fd,
                SOUND_MIXER_READ_STEREODEVS,
                &stereomask
            )
        )
            return(-1);

        if(value1 > 1)   
            value1 = 1;  
        if(value1 < 0)
            value1 = 0;

        if(value2 > 1)
            value2 = 1;
        if(value2 < 0)
            value2 = 0;

        value1 = value1 * 100 / 255;
        value2 = value2 * 100 / 255;

        /* Sanitize left and right for non-stereo mixer device. */
        if((value1 != value2) && !((1 << i) & stereomask))
        {
            /* Average the two values. */
            avg = (value1 + value2) / 2;
            value1 = avg;
            value2 = avg;
        }

        left = (unsigned int)(value1 * (double)255);
        right = (unsigned int)(value2 * (double)255);

        packed_value = (right << 8) + left;

        return(  
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetRec(int fd, double value)
{
        int i, packed_value;
        
          
        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "rec");
        if(i < 0)
            return(-1);

        if(value > 1)  
            value = 1;
        if(value < 0)
            value = 0;

        packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );        
}

int MixerSetSpeaker(int fd, double value)
{
        int i, packed_value;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "speaker");
        if(i < 0)
            return(-1);

        if(value > 1)
            value = 1; 
        if(value < 0)
            value = 0;

	value = value * 100 / 255;
        packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );        
}

int MixerSetSynth(int fd, double value1, double value2)
{
        int i, packed_value;
        int stereomask;
        double avg;
        int left, right;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "synth");
        if(i < 0)
            return(-1);

        /* Get mixer stereo mask. */
        if(
            ioctl(
                fd,
                SOUND_MIXER_READ_STEREODEVS,
                &stereomask
            )
        )
            return(-1);

        if(value1 > 1)   
            value1 = 1;  
        if(value1 < 0)
            value1 = 0;

        if(value2 > 1)
            value2 = 1;
        if(value2 < 0)
            value2 = 0;

        value1 = value1 * 100 / 255;
        value2 = value2 * 100 / 255;

        /* Sanitize left and right for non-stereo mixer device. */
        if((value1 != value2) && !((1 << i) & stereomask))
        {
            /* Average the two values. */
            avg = (value1 + value2) / 2;
            value1 = avg;
            value2 = avg;
        }

        left = (unsigned int)(value1 * (double)255);
        right = (unsigned int)(value2 * (double)255);

        packed_value = (right << 8) + left;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetTreble(int fd, double value)
{
        int i, packed_value;


        if(fd < 0)
            return(-1);

        /* Get mixer device number. */
        i = GET_MIXER_NUM_BY_NAME(fd, "treble");
        if(i < 0)
            return(-1);

	if(value > 1)
	    value = 1;
	if(value < 0)
	    value = 0;

	value = value * 100 / 255;
	packed_value = value * 255;

        return(
            ioctl(fd, MIXER_WRITE(i), &packed_value)
        );
}

int MixerSetVolume(int fd, double value1, double value2)
{
	int i, packed_value;
	int stereomask;
	double avg;
	int left, right;


        if(fd < 0)
            return(-1);

	/* Get mixer device number. */
	i = GET_MIXER_NUM_BY_NAME(fd, "vol");
	if(i < 0)
	    return(-1);

        /* Get mixer stereo mask. */
	if(
	    ioctl(
		fd,
		SOUND_MIXER_READ_STEREODEVS,
		&stereomask
	    )
	)
            return(-1);


	if(value1 < 0)
	    value1 = 0;
	if(value1 > 1)
	    value1 = 1;

        if(value2 < 0)
            value2 = 0;
        if(value2 > 1)
            value2 = 1;

	value1 = value1 * 100 / 255;
	value2 = value2 * 100 / 255;

        /* Sanitize left and right for non-stereo mixer device. */
        if((value1 != value2) && !((1 << i) & stereomask))
        {
            /* Average the two values. */
            avg = (value1 + value2) / 2;
            value1 = avg;
            value2 = avg;
        }

	left = (unsigned int)(value1 * (double)255);
	right = (unsigned int)(value2 * (double)255);

	packed_value = (right << 8) + left;

	return(
	    ioctl(fd, MIXER_WRITE(i), &packed_value)
	);
}


/* ******************************************************************* */

/*
 *	Sets mixer device mixer_code to the specified value.
 *
 *	Returns -1 on error or 0 on success.
 */
int MixerSet(
	int fd,
	int mixer_code,
	double value1,
	double value2,
	double value3,
	double value4,
	double value5
)
{
	int status = -2;


        /*
         *      The fd givin is the fd for the sound device.
         *      The mixer can be adjusted with the sound device
         *      fd and does not require that the mixer device be
         *      opened as long as the sound device is opened.
         */
        if(fd < 0)
            return(-1);

        
	switch(mixer_code)
	{
	  case MixerCodeBass:
            status = MixerSetBass(fd, value1);
            break;

          case MixerCodeCD:
            status = MixerSetCD(fd, value1, value2);
            break;

          case MixerCodeGainIn:
            status = MixerSetGainIn(fd, value1, value2);
            break;

          case MixerCodeGainOut:
            status = MixerSetGainOut(fd, value1, value2);
            break;

          case MixerCodeLine:
            status = MixerSetLine(fd, value1);
            break;

          case MixerCodeLine1:
            status = MixerSetLine1(fd, value1);
            break;

          case MixerCodeLine2:
            status = MixerSetLine2(fd, value1);
            break;

          case MixerCodeLine3:
            status = MixerSetLine3(fd, value1);
            break;

          case MixerCodeMic:
            status = MixerSetMic(fd, value1, value2);
            break;

          case MixerCodeMix:
            status = MixerSetMix(fd, value1);
            break;

          case MixerCodePCM:
            status = MixerSetPCM(fd, value1, value2);
            break;

          case MixerCodePCM2:
            status = MixerSetPCM2(fd, value1, value2);
            break;

          case MixerCodeRec:
            status = MixerSetRec(fd, value1);
            break;

          case MixerCodeSpeaker:
            status = MixerSetSpeaker(fd, value1);
            break;

          case MixerCodeSynth:
            status = MixerSetSynth(fd, value1, value2);
            break;

          case MixerCodeTreble:
            status = MixerSetTreble(fd, value1);
            break;

	  case MixerCodeVolume:
	    status = MixerSetVolume(fd, value1, value2);  
	    break;
	}



        return(status);
}
