/*
                  Restart Configuration File

	Functions:

	int RCLoadFromFile(char *filename)

	---

 */

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include "../include/cfgfmt.h"
#include "../include/string.h"
#include "../include/strexp.h"
#include "../include/fio.h"

#include "ymode.h"
#include "yiff.h"



#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))



extern int antishift(int in);


int RCLoadFromFile(char *filename)
{
	int status;
        char *strptr, *strptr2;

        FILE *fp;
        off_t filesize;
        struct stat stat_buf;

        char parm[CFG_PARAMETER_MAX];
        char val[CFG_VALUE_MAX];
        int lines_read = 0;

	YMode *ymode_ptr;
	SoundPath *sp_ptr;


	/* Check if filename exists. */
	if(filename == NULL)
	    return(-1);
	if(stat(filename, &stat_buf))
	{
	    fprintf(stderr, "%s: No such file.\n", filename);
	    return(-1);
	}

	/* Get size of file. */
        filesize = stat_buf.st_size;

        /* Open filename. */
        fp = fopen(filename, "r");
        if(fp == NULL)
        {
            fprintf(stderr, "%s: Cannot open.\n", filename);
            return(-1);
        }


	/* ************************************************************* */
	/* Free any allocated resources as needed. */




	/* ******************************************************** */

	strptr = NULL;

        while(1)
        {
	    /* Free previous line and allocate/read next line. */
            free(strptr); strptr = NULL;
            strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
            if(strptr == NULL) break;
            lines_read++;

            /* Fetch parameter. */
            strptr2 = StringCfgParseParm(strptr);
            if(strptr2 == NULL) continue;
            strncpy(parm, strptr2, CFG_PARAMETER_MAX);
            parm[CFG_PARAMETER_MAX - 1] = '\0';

            /* Fetch value. */
            strptr2 = StringCfgParseValue(strptr);
            if(strptr2 == NULL) strptr2 = "0";  /* Set it to "0" if NULL. */
            strncpy(val, strptr2, CFG_VALUE_MAX);
            val[CFG_VALUE_MAX - 1] = '\0';


	    /* VersionMajor */
            if(!strcmp(parm, "VersionMajor"))
            {

	    }
            /* VersionMinor */
            else if(!strcmp(parm, "VersionMinor"))
            {

            }

	    /* Port */
            else if(!strcmp(parm, "Port"))
            {
		option.port = atoi(val);
	    }
	    /* RefreshInterval */
            else if(!strcmp(parm, "RefreshInterval"))
            {
		option.refresh_int.ms = atol(val) / 1000;
                option.refresh_int.us = atol(val) % 1000;
            }
	    /* Device */
	    else if(!strcmp(parm, "Device"))
	    {
		strncpy(
		    fname.device,
		    val,
		    PATH_MAX + NAME_MAX
		);
                if(stat(fname.device, &stat_buf))
                {
                    fprintf(stderr,
   "%s: Line %i: Warning: No such device `%s'.\n",
                            filename,
                            lines_read,
                            fname.device
                    );
                }
	    }

            /* **************************************************** */
	    /* BeginMode */
            else if(!strcmp(parm, "BeginYMode") ||
                    !strcmp(parm, "BeginMode")
	    )
            {
		status = YModeAllocate(NULL);
		ymode_ptr = YModeGetPtr(status);
		if(ymode_ptr == NULL)
                    continue;

                while(1)
                {
                    /* Free previous line and allocate/read next line. */
                    free(strptr); strptr = NULL;
                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
                    if(strptr == NULL) break;
                    lines_read++;
                    
                    /* Fetch parameter. */
                    strptr2 = StringCfgParseParm(strptr);
                    if(strptr2 == NULL) continue;
                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
                    parm[CFG_PARAMETER_MAX - 1] = '\0';
                        
                    /* Fetch value. */
                    strptr2 = StringCfgParseValue(strptr);  
                    if(strptr2 == NULL) strptr2 = "0";
                    strncpy(val, strptr2, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';


                    /* Name */
                    if(!strcmp(parm, "Name"))
                    {
			free(ymode_ptr->name);
			ymode_ptr->name = StringCopyAlloc(val);
		    }
                    /* Cycle */
                    else if(!strcmp(parm, "Cycle"))
                    {
                        ymode_ptr->cycle.ms = atol(val) / 1000;
                        ymode_ptr->cycle.us = atol(val) % 1000;
                    }
                    /* WriteAhead */
                    else if(!strcmp(parm, "WriteAhead"))
                    {
                        ymode_ptr->write_ahead.ms = atol(val) / 1000;
                        ymode_ptr->write_ahead.us = atol(val) % 1000;
                    }
                    /* SampleSize */
                    else if(!strcmp(parm, "SampleSize"))
                    {
                        ymode_ptr->sample_size = atoi(val);

			if(ymode_ptr->sample_size == 16)
			    ymode_ptr->sample_size = 16;
			else
			    ymode_ptr->sample_size = 8;
                    }
                    /* Channels */
                    else if(!strcmp(parm, "Channels"))
                    {
                        ymode_ptr->channels = atoi(val);

                        if(ymode_ptr->channels == 2)
                            ymode_ptr->channels = 2;
                        else
                            ymode_ptr->channels = 1;
                    }
                    /* SampleRate */
                    else if(!strcmp(parm, "SampleRate"))
                    {
                        ymode_ptr->sample_rate = atoi(val);
                    }
#ifdef SOUND_EXPERIMENTAL
                    /* AllowFragmenting */
                    else if(!strcmp(parm, "AllowFragmenting"))
                    {
                        ymode_ptr->allow_fragments = StringIsYes(val);
                    }
                    /* Fragments */
                    else if(!strcmp(parm, "Fragments"))
                    {
                        ymode_ptr->num_fragments = atoi(val);
                    }
                    /* FragmentSize */
                    else if(!strcmp(parm, "FragmentSize"))
                    {
			/*   Note: On file, it's bytes, in memory it's
                         *   shifts.
                         */
                        ymode_ptr->fragment_size = antishift(atoi(val) - 1);
                    }
#endif  /* SOUND_EXPERIMENTAL */
                    /* FlipStereo */
                    else if(!strcmp(parm, "FlipStereo"))
                    {
                        ymode_ptr->flip_stereo = StringIsYes(val);
                    }
                    /* Direction */
                    else if(!strcmp(parm, "Direction"))
                    {
			if(!strcmp(val, "Record"))
			    ymode_ptr->direction = AUDIO_DIRECTION_RECORD;
			else
                            ymode_ptr->direction = AUDIO_DIRECTION_PLAY;
                    }

                    /* EndMode */
                    else if(!strcmp(parm, "EndYMode") ||
                            !strcmp(parm, "EndMode")
		    )
                    {
                        break;
                    }

                    /* Unknown parameter. */
                    else
                    { 
                        fprintf(stderr,
                            "%s: Line %i: Unknown parameter: %s\n",
                            filename,
                            lines_read,
                            parm
                        );
                        continue;
                    }

                }
            }

            /* **************************************************** */
            /* BeginSoundPath */
            else if(!strcmp(parm, "BeginSoundPath"))
            {
                status = SoundPathAllocate();
                sp_ptr = SoundPathGetPtr(status);
                if(sp_ptr == NULL)
                    continue;

                while(1)
                {
                    /* Free previous line and allocate/read next line. */
                    free(strptr); strptr = NULL;
                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
                    if(strptr == NULL) break;
                    lines_read++;

                    /* Fetch parameter. */
                    strptr2 = StringCfgParseParm(strptr);
                    if(strptr2 == NULL) continue;
                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
                    parm[CFG_PARAMETER_MAX - 1] = '\0';

                    /* Fetch value. */
                    strptr2 = StringCfgParseValue(strptr);
                    if(strptr2 == NULL) strptr2 = "0";
                    strncpy(val, strptr2, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';  


                    /* Path */   
                    if(!strcmp(parm, "Path"))
                    {
                        free(sp_ptr->path);
                        sp_ptr->path = StringCopyAlloc(val);
                    }
                    /* EndSoundPath */
                    else if(!strcmp(parm, "EndSoundPath"))
                    {
                        break;
                    }
                    /* Unknown parameter. */  
                    else
                    {
                        fprintf(stderr,
                            "%s: Line %i: Unknown parameter: %s\n",
                            filename,
                            lines_read,
                            parm
                        );
                        continue;
                    } 
                }
            }

            /* Unknown parameter. */
            else
            {
                fprintf(stderr, "%s: Line %i: Unknown parameter: %s\n",
                    filename,
                    lines_read,
                    parm
                );
            }
	}


	/* Close the file. */
	fclose(fp); fp = NULL;


	return(0);
}
