/*
	Functions:

	int antishift(int in)
	void CRSanitize(char *s)
	void CON_NOTIFY(YConnectionNumber con_num, char *mesg)

	void YiffSignalHandler(int sig)
	void YiffResetMixer()

	int YiffInit(int argc, char **argv)
	void YiffShutdown()

	void YiffDoConnectionCommand(
		YConnectionNumber con_num,
		char *cmd
	)
	int YiffCheckNewConnection(int socket)
	void YiffCloseConnection(YConnectionNumber con_num)
	void YiffManageConnection(YConnectionNumber con_num)
	void YiffManageConnections()

	int YiffCreatePlaystack(
		char *path,
		YConnectionNumber owner,
		YID yid,
		YDataPosition pos,
		Coefficient volume_left,
                Coefficient volume_right,
		int repeats
	)
	void YiffManageSound()

	void YiffUpdateTimmers()
	void YiffResetTimmers()

	int main(int argc, char *argv[])

	---

 */

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>		/* For inet_ntoa. */
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "../include/string.h"
#include "../include/strexp.h"
#include "../include/ynetwork.h"
#include "ymode.h"
#include "yhost.h"
#include "mixer.h"
#include "yiff.h"
#include "config.h"

/* From rcfile.c. */
extern int RCLoadFromFile(char *filename);


#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))

#define MINSC(a,b)	((a < b) ? (char)127 : (char)(a))
#define MAXSC(a,b)	((a > b) ? (char)-128 : (char)(a))



/*
 *	Local timmings:
 */
time_t cycle_ahead_ms;		/* In milliseconds. */
time_t next_sound_delta_us;	/* In microseconds. */



/*
 *	Local functions:
 */
int antishift(int in);
void CRSanitize(char *s);
void CON_NOTIFY(YConnectionNumber con_num, char *mesg);


/*
 *	Anti shift macro:
 */
int antishift(int in)
{
        int s;
        int i = in;

        for(s = 0; i; i >>= 1)
            s++;


        return(s);
}


/*
 *	Replaces occurances of \r with \n.
 */
void CRSanitize(char *s)
{
	if(s == NULL)
	    return;

	s = strchr(s, '\r');
	while(s != NULL)
	{
	    *s = '\n';
	    s = strchr(s, '\r');
	}

	return;
}


/*
 *	Sends message to connection.
 */
void CON_NOTIFY(YConnectionNumber con_num, char *mesg)
{
        struct timeval timeout;
        fd_set writefds;

	int len, bytes_written;
	YConnection *con_ptr;


	/* Error checks. */
	if(mesg == NULL)
	    return;
	if(!ConnectionIsConnected(con_num))
	    return;

	con_ptr = yconnection[con_num];
	len = strlen(mesg);


        timeout.tv_sec = 3;		/* Timeout set to 3 seconds. */
        timeout.tv_usec = 0;
        FD_ZERO(&writefds);
        FD_SET(con_ptr->socket, &writefds);
        select(con_ptr->socket + 1, NULL, &writefds, NULL, &timeout);

        if(!FD_ISSET(con_ptr->socket, &writefds))
            return;


	bytes_written = send(
	    con_ptr->socket,
	    mesg,
	    len,
	    0
	);


	return;
}



/*
 *	Signal handler.
 */
void YiffSignalHandler(int sig)
{
	switch(sig)
	{
	  case SIGINT:
            runlevel = 1;
            break;

          case SIGKILL:  
            runlevel = 1;
            break;

          case SIGPIPE:
            signal(SIGPIPE, YiffSignalHandler);
            break;

          case SIGSEGV:
            runlevel = 1;
            break;

	  case SIGTERM:
	    runlevel = 1;
	    break;

	  default:
	    break;
	}

	return;
}


/*
 *	Procedure to reset mixer values.
 */
void YiffResetMixer()
{
	int fd;


	if(recorder == NULL)
	    return;

	fd = recorder->audio.fd;
	if(fd < 0)
	    return;

	MixerSet(fd, MixerCodeBass,    0.75, 0.75, 0, 0, 0);
        MixerSet(fd, MixerCodeCD,      0.80, 0.80, 0, 0, 0);
        MixerSet(fd, MixerCodeGainIn,  0.80, 0.80, 0, 0, 0);
        MixerSet(fd, MixerCodeGainOut, 0.80, 0.80, 0, 0, 0);
/*
        MixerSet(fd, MixerCodeLine,    0.80, 0.80, 0, 0, 0);
        MixerSet(fd, MixerCodeLine1,   0.80, 0.80, 0, 0, 0);
        MixerSet(fd, MixerCodeLine2,   0.80, 0.80, 0, 0, 0);
        MixerSet(fd, MixerCodeLine3,   0.80, 0.80, 0, 0, 0);
 */
        MixerSet(fd, MixerCodeMic,     0.20, 0.20, 0, 0, 0);
        MixerSet(fd, MixerCodeMix,     0.80, 0.80, 0, 0, 0);
        MixerSet(fd, MixerCodePCM,     0.80, 0.80, 0, 0, 0);
/*
        MixerSet(fd, MixerCodePCM2,    0.80, 0.80, 0, 0, 0);
 */
        MixerSet(fd, MixerCodeRec,     0.20, 0.20, 0, 0, 0);
        MixerSet(fd, MixerCodeSpeaker, 0.80, 0.80, 0, 0, 0);
        MixerSet(fd, MixerCodeSynth,   0.80, 0.80, 0, 0, 0);
        MixerSet(fd, MixerCodeTreble,  0.75, 0.75, 0, 0, 0);
        MixerSet(fd, MixerCodeVolume,  0.80, 0.80, 0, 0, 0);


	return;
}



/*
 *	Initializes the program, should call this function
 *	at the very start.
 */
int YiffInit(int argc, char **argv)
{
	int i, n, status;
	struct sockaddr_in addr;
	char cwd[PATH_MAX];
	struct stat stat_buf;

	int override_device = 0;
	char device[PATH_MAX + NAME_MAX];
	int override_port = 0;
	int port = -1;


	getcwd(cwd, PATH_MAX);
	cwd[PATH_MAX - 1] = '\0';


	/* Reset global variables. */
	option.debug = False;
	option.port = DEF_PORT;
	option.sample_size = DEF_SAMPLE_SIZE;
        option.channels = DEF_CHANNELS;
        option.sample_rate = DEF_SAMPLE_RATE;

#ifdef SOUND_EXPERIMENTAL
        /*
         *  There is a new ioctl called SNDCTL_DSP_SETFRAGMENT. It
         *  accepts an int parameter which has format 0x00nn00ss where
         *  the nn is max number of buffer fragments (between 0x02 and 0xff)
         *  and the ss gives indirectly the size of a buffer fragment
         *  (fragment_size = (1 << ss)). Valid sizes are between
         *  (ss=0x07 -> 128 bytes and ss=0x11 (17 dec) -> 128k).
         *
         *      Linux documentation suggestion: 0x00020009
         *      XBlast Sound Server: 0x0004000a
         */
	option.allow_fragments = True;
	option.num_fragments = 0x02;
        option.fragment_size = 0x0b;
#endif	/* SOUND_EXPERIMENTAL */

	option.flip_stereo = False;
	option.direction = AUDIO_DIRECTION_PLAY;
	option.cycle_set = CYCLE_SET_USER;

	option.refresh_int.ms = 60000;		/* 1 minute. */
	option.refresh_int.us = 0;

        strncpy(fname.rcfile, DEF_RCFILE_NAME, PATH_MAX + NAME_MAX);
        fname.rcfile[PATH_MAX + NAME_MAX - 1] = '\0';
	strncpy(fname.device, DEF_DEVICE, PATH_MAX + NAME_MAX);
	fname.device[PATH_MAX + NAME_MAX - 1] = '\0';


	stats.start_time = time(NULL);
	stats.cycle_load = 0.00;


	/* Delete any existing sound paths. */
	SoundPathDeleteAll();

	/* Allocate current directory as first sound path. */
	i = SoundPathAllocate();
	if(SoundPathIsAllocated(i))
	{
	    free(soundpath[i]->path);
	    soundpath[i]->path = StringCopyAlloc(cwd);
	}


	/* Delete any existing yhosts. */
	YHostDeleteAll();

	YHostAllocate("127.0.0.1");


        /* ****************************************************** */
        /* Parse arguments. */
	for(i = 1; i < argc; i++)
	{
	    if(argv[i] == NULL)
		continue;

	    /* Print help. */
	    if(strpfx(argv[i], "--h") ||
               strpfx(argv[i], "-h") ||
               strpfx(argv[i], "-?")
	    )
	    {
		printf(YIFF_HELP);
		return(-4);
	    }
            /* Print version. */
            else if(strpfx(argv[i], "--ver") ||
                    strpfx(argv[i], "-ver")
            )
            {
                printf("%s Version %s\n", YIFF_NAME, YIFF_VERSION);
                return(-4);
            }
            /* Device. */
            else if(strpfx(argv[i], "--d") ||
                    strpfx(argv[i], "-d")
            )
            {
                i++;
                if(i < argc)
                {
		    strncpy(device, argv[i], PATH_MAX + NAME_MAX);
		    device[PATH_MAX + NAME_MAX - 1] = '\0';

		    override_device = 1;
		}
                else
                {
                    fprintf(stderr,
                        "%s: Requires argument.\n",
                        argv[i - 1]
                    );
                }
	    }
            /* Port. */
            else if(strpfx(argv[i], "--po") ||
                    strpfx(argv[i], "-po")
            )
            {
                i++;
                if(i < argc)
                {
		    port = atoi(argv[i]);
                    override_port = 1;
                }   
                else
                {
                    fprintf(stderr,
                        "%s: Requires argument.\n",
                        argv[i - 1]
                    );
                }
            }
            /* Sound path. */
            else if(strpfx(argv[i], "--pa") ||
                    strpfx(argv[i], "-pa")
            )
            {
                i++;
                if(i < argc)
                {
		    n = SoundPathAllocate();
		    if(SoundPathIsAllocated(n))
		    {
			free(soundpath[n]->path);
			soundpath[n]->path = StringCopyAlloc(argv[i]);

			if(stat(soundpath[n]->path, &stat_buf))
			{
			    fprintf(stderr,
 "%s: Warning: No such directory.\n",
				soundpath[n]->path
			    );
			}
			else
			{
			    if(!S_ISDIR(stat_buf.st_mode))
                                fprintf(stderr,
 "%s: Warning: Not a directory.\n",
				    soundpath[n]->path
                                );
			}
		    }
                }
                else
                {
                    fprintf(stderr,
                        "%s: Requires argument.\n",
                        argv[i - 1]
                    );
                }
            }
	    /* All else, it's the rcfile. */
	    else
	    {
		strncpy(fname.rcfile, argv[i], PATH_MAX + NAME_MAX);
		fname.rcfile[PATH_MAX + NAME_MAX - 1] = '\0';
	    }
	}

        /* ****************************************************** */
        /* Load configuration. */
	status = RCLoadFromFile(fname.rcfile);
	if(status)
	    return(-1);

	/* Need to set options to match default mode 0. */
	n = 0;
	if(YModeIsAllocated(n))
	{
            /* Set new parameters. */
            option.sample_size = ymode[n]->sample_size;
            option.channels = ymode[n]->channels;
            option.sample_rate = ymode[n]->sample_rate;

#ifdef SOUND_EXPERIMENTAL
            option.allow_fragments = ymode[n]->allow_fragments;
            option.num_fragments = ymode[n]->num_fragments;  
            option.fragment_size = ymode[n]->fragment_size;  
#endif  /* SOUND_EXPERIMENTAL */

            option.direction = ymode[n]->direction;

            option.cycle_set = CYCLE_SET_USER;
            option.cycle.ms = ymode[n]->cycle.ms;
            option.cycle.us = ymode[n]->cycle.us;
	}	

	/* Override device? */
	if(override_device)
	{
	    strncpy(fname.device, device, PATH_MAX + NAME_MAX);
	    fname.device[PATH_MAX + NAME_MAX - 1] = '\0';
	}
        /* Override port? */
        if(override_port)
        {
            option.port = port;
        }


        /* ****************************************************** */
        /* Set signals to watch. */
        signal(SIGINT, YiffSignalHandler);
        signal(SIGKILL, YiffSignalHandler);
        signal(SIGPIPE, YiffSignalHandler);
        signal(SIGSEGV, YiffSignalHandler);
	signal(SIGTERM, YiffSignalHandler);


        /* ****************************************************** */
	/* Reset timmers. */
	YiffResetTimmers();


        /* ****************************************************** */
	/* Allocate recorder structure and initialize sound. */
	recorder = (Recorder *)calloc(1, sizeof(Recorder));
	if(recorder == NULL)
	    return(-1);
	/* Initialize sound. */
	if(
	    YSoundInit(
		argc,
		argv,
		recorder
	    )
	)
	    return(-1);

	/* User set cycle? */
        if(option.cycle_set == CYCLE_SET_USER)
	{
	    recorder->audio.cycle.ms = option.cycle.ms;
            recorder->audio.cycle.us = option.cycle.us;

            recorder->audio.compensated_cycle.ms = option.cycle.ms;
            recorder->audio.compensated_cycle.us = option.cycle.us;
	}

	/* Reset mixer values. */
	YiffResetMixer();


	/* ****************************************************** */
	/* Set up listening socket. */
	listen_socket = socket(AF_INET, SOCK_STREAM, 0);
	if(listen_socket < 0)
            return(-1);

        addr.sin_family = AF_INET;
        addr.sin_port = htons(option.port);
        addr.sin_addr.s_addr = INADDR_ANY;
        bzero(&addr.sin_zero, 8);

        /* Bind the socket. */
        status = bind(
	    listen_socket,
            (struct sockaddr *)&addr,
            sizeof(struct sockaddr)
	);
	if(status)
            return(-1);

	status = listen(listen_socket, LISTEN_SOCKET_BACKLOG);
	if(status)
            return(-1);

        /* Set listening port socket to nonblocking. */
        fcntl(listen_socket, F_SETFL, O_NONBLOCK);


	/* ********************************************************** */
	/* Update timmers (to help sync audio device). */
	YiffUpdateTimmers();

	/* Need to sync audio device right after initialization. */
	YSoundSync(recorder, 0);

#ifdef __linux__
	/* Calculate cycle_ahead_ms. */
	if(recorder != NULL)
	{
	    if(recorder->audio.allow_fragments)
	    {
	        cycle_ahead_ms = 1.5 *
		    recorder->audio.compensated_cycle.ms;

if(option.debug)
    fprintf(stderr, "cycle_ahead_ms: %ld ms\n", cycle_ahead_ms);
	    }
	}
#endif	/* __linux__ */
	next_sound_delta_us = 0;

	/* schedual next refresh. */
        next.refresh.ms = cur_time.ms + option.refresh_int.ms;
	next.refresh.us = 0;


	return(0);
}



void YiffShutdown()
{
	/* Free all sound paths. */
	SoundPathDeleteAll();

	/* Free all play stacks. */
	PlayStackDeleteAll();

        /* Deallocate recorder structure and shutdown sound. */
	YSoundShutdown(recorder);
	free(recorder);
	recorder = NULL;


	/* Close listening socket. */
	if(listen_socket > -1)
	{
	    close(listen_socket);
	    listen_socket = -1;
	}

	/* Disconnect all connections. */
	ConnectionDeleteAll();

	/* Free all yhosts. */
	YHostDeleteAll();

	/* Free all ymodes. */
	YModeDeleteAll();


	return;
}


/*
 *	Handle connection's command.
 */
void YiffDoConnectionCommand(
	YConnectionNumber con_num, 
	char *cmd
)
{
	int i, n;
	char *strptr;

	char **strv;
	int strc;

	YID yid;
	YConnection *con_ptr;
	char sndbuf[1024];


	con_ptr = ConnectionGetPtr(con_num);
	if((con_ptr == NULL) ||
           (cmd == NULL)
	)
	    return;

	strv = strexp(cmd, &strc);
	if((strv == NULL) ||
           (strc <= 0)
	)
	    return;


	/* ********************************************************* */

        /*
         *    Change mode:
         *
         *      YNET_CMD_CHANGEMODE sample_size channels sample_rate
         *      direction allow_fragmenting num_fragments fragment_size_bytes
         */
        if(!strcmp(strv[0], YNET_CMD_CHANGEMODE))
        {
/* Check if connection has change mode access. */   
            if(strc >= 8)
            {
                /* Delete all playstacks. */
                for(i = 0; i < total_playstacks; i++)
                {
                    if(playstack[i] == NULL)
                        continue;

                    sprintf(sndbuf, "%s %i\n",
                        YNET_CMD_KILL,
                        i + 1
                    );
                    CON_NOTIFY(playstack[i]->owner, sndbuf);

                    PlayStackDelete(i);
                }

                /* Close sound device. */
                YSoundShutdown(recorder);
                free(recorder);
                recorder = NULL;

                /* Set new parameters. */
                option.sample_size = atoi(strv[1]);
                option.channels = atoi(strv[2]);
                option.sample_rate = atoi(strv[3]);
                option.direction = atoi(strv[4]);
#ifdef SOUND_EXPERIMENTAL
		option.allow_fragments = (atoi(strv[5])) ? True : False;
		option.num_fragments = atoi(strv[6]);
		option.fragment_size = antishift(atoi(strv[7]) - 1);
#endif	/* SOUND_EXPERIMENTAL */

                /* Restart sound device. */
                recorder = (Recorder *)calloc(1, sizeof(Recorder));
                if(recorder == NULL)
                    return;
                YSoundInit(0, NULL, recorder);

		/* Notify all connections of mode change. */
	        sprintf(sndbuf, "%s\n",
                    YNET_CMD_CHANGEMODE
                );
		for(i = 0; i < total_yconnections; i++)
		{
                    CON_NOTIFY(i, sndbuf);
		}
            }
        }
        /*
         *    Change mode preset:
         *
         *      YNET_CMD_CHANGEMODEPRESET preset_mode_name
         */
        else if(!strcmp(strv[0], YNET_CMD_CHANGEMODEPRESET))
	{
/* Check if connection has change mode access. */
	    if(strc >= 2)
	    {
	        n = YModeMatch(strv[1]);
		if(YModeIsAllocated(n))
		{
                    /* Delete all playstacks. */
                    for(i = 0; i < total_playstacks; i++)
                    {
                        if(playstack[i] == NULL)
                            continue;

                        sprintf(sndbuf, "%s %i\n",
                            YNET_CMD_KILL,
                            i + 1
                        );
                        CON_NOTIFY(playstack[i]->owner, sndbuf);

                        PlayStackDelete(i);
                    }

                    /* Close sound device. */
                    YSoundShutdown(recorder);
                    free(recorder);
                    recorder = NULL;

                    /* Set new parameters. */
                    option.sample_size = ymode[n]->sample_size;
                    option.channels = ymode[n]->channels;
                    option.sample_rate = ymode[n]->sample_rate;

#ifdef SOUND_EXPERIMENTAL
                    option.allow_fragments = ymode[n]->allow_fragments;
                    option.num_fragments = ymode[n]->num_fragments;
                    option.fragment_size = ymode[n]->fragment_size;
#endif  /* SOUND_EXPERIMENTAL */

                    option.direction = ymode[n]->direction;

                    option.cycle_set = CYCLE_SET_USER;
                    option.cycle.ms = ymode[n]->cycle.ms;
                    option.cycle.us = ymode[n]->cycle.us;


                    /* Restart sound device. */
                    recorder = (Recorder *)calloc(1, sizeof(Recorder));
                    if(recorder == NULL)
                        return;
                    YSoundInit(0, NULL, recorder);

		    /* Set cycle on recorder. */
                    recorder->audio.cycle.ms = option.cycle.ms;
                    recorder->audio.cycle.us = option.cycle.us;

                    recorder->audio.compensated_cycle.ms =
                        recorder->audio.cycle.ms;
                    recorder->audio.compensated_cycle.us = 
                        recorder->audio.cycle.us;


                    /* Notify all connections of mode change. */
                    sprintf(sndbuf, "%s %s\n",
                        YNET_CMD_CHANGEMODEPRESET,
			strv[1]		/* Preset mode name. */
                    );
                    for(i = 0; i < total_yconnections; i++)
                    {
                        CON_NOTIFY(i, sndbuf);
                    }
		}
	    }
	}
	/*
         *    Cycle:
	 *
	 *	YNET_CMD_CYCLE cycle_in_us
	 */
        else if(!strcmp(strv[0], YNET_CMD_CYCLE))
        {
	    if((recorder != NULL) && (strc > 1))
	    {
		if(recorder->audio.cycle_set == CYCLE_SET_USER)
		{
                    recorder->audio.cycle.ms = atol(strv[1]) / 1000;
                    recorder->audio.cycle.us = atol(strv[1]) % 1000;

		    recorder->audio.compensated_cycle.ms =
			recorder->audio.cycle.ms;
                    recorder->audio.compensated_cycle.us =
			recorder->audio.cycle.us;

		    /* Notify all connections of cycle change. */
                    sprintf(sndbuf, "%s %i\n",
			YNET_CMD_CYCLE, 
			(int)((recorder->audio.compensated_cycle.ms *
			    1000) +
                            recorder->audio.compensated_cycle.us
			)
		    );
                    for(i = 0; i < total_yconnections; i++)
                    {
                        CON_NOTIFY(i, sndbuf);
                    }
		}
	    }
	}
	/*
	 *    Debug:
	 *
	 *	"debug" 0|1 value
	 */
	else if(!strcmp(strv[0], "debug"))
	{
	    if(strc >= 2)
	    {
		if(atoi(strv[1]))
		    option.debug = True;
		else
		    option.debug = False;
	    }
	    else
	    {
                sprintf(sndbuf, "%s Debugging is currently %s.\n", 
                    "debug",
                    (option.debug) ? "on" : "off"
                );
                CON_NOTIFY(con_num, sndbuf);
	    }
	}
        /*
         *    Disconnect:
         *
         *      YNET_CMD_DISCONNECT
         */
        else if(!strcmp(strv[0], YNET_CMD_DISCONNECT))
        {
            YiffCloseConnection(con_num);
        }
        /*
         *    Host:
         *
         *      YNET_CMD_HOST 0|1 "123.123.123.123"
         */
        else if(!strcmp(strv[0], YNET_CMD_HOST))
        {
	    /* Only localhost connection may add or remove hosts. */
	    if(!strcmp(con_ptr->host, "127.0.0.1"))
	    {
		if(strc >= 3)
	        {
		    /* Add or remove host? */
		    if(atoi(strv[1]))
		    {
			YHostAllocate(strv[2]);

                        sprintf(sndbuf, "%s %i %s\n",
			    YNET_CMD_HOST,
			    1,
			    strv[2]
                        );
                        CON_NOTIFY(con_num, sndbuf);
		    }
		    else
		    {
			if(!strcmp(strv[2], "127.0.0.1"))
			{
/* Notify connection of error, cannot remove localhost from hosts list. */
			}
			else
			{
			    YHostDeleteByHost(strv[2]);

                            sprintf(sndbuf, "%s %i %s\n",
                                YNET_CMD_HOST,
                                0,
                                strv[2]
                            );
                            CON_NOTIFY(con_num, sndbuf);
			}
		    }
		}
		else
		{
/* Notify connection of error, no argument given. */
		}
	    }
	    else
	    {
/* Notify connection of error, it is not allowed to add or remove hosts. */
	    }
        }
	/*
	 *    Kill:
         *
         *      YNET_CMD_KILL yid
         */
        else if(!strcmp(strv[0], YNET_CMD_KILL))
        {
	    if(strc >= 2)
	    {
	        yid = atol(strv[1]);
                if(yid != YIDNULL)
                {
		    for(i = 0; i < total_playstacks; i++)
		    {
			if(playstack[i] == NULL) continue;
			if(playstack[i]->owner != con_num) continue;
			if(playstack[i]->yid == YIDNULL) continue;
			if(playstack[i]->yid != yid) continue;

			PlayStackDelete(i);

			sprintf(sndbuf, "%s %i\n",
			    YNET_CMD_KILL,
			    yid
			);
			CON_NOTIFY(con_num, sndbuf);
		    }
		}
	    }
	}
	/*
	 *    Mixer:
	 *
	 *	YNET_CMD_MIXER mixer_code value1 value2 ... value5
	 */
        else if(!strcmp(strv[0], YNET_CMD_MIXER))
	{
	    if((recorder != NULL) && (strc >= 2))
	    {
		MixerSet(
		    recorder->audio.fd,
		    atoi(strv[1]),
		    (strc >= 3) ? atof(strv[2]) : 0.0,
                    (strc >= 4) ? atof(strv[3]) : 0.0,
                    (strc >= 5) ? atof(strv[4]) : 0.0,
                    (strc >= 6) ? atof(strv[5]) : 0.0,
                    (strc >= 7) ? atof(strv[6]) : 0.0
		);
	    }
	}
	/*
	 *    Play:
	 *
	 *	YNET_CMD_PLAY yid filename pos volume_left volume_right
	 *	repeats
	 */
	else if(!strcmp(strv[0], YNET_CMD_PLAY))
	{
	    if(strc >= 3)
	    {
                strptr = SoundPathCompletePath(strv[2]);

                YiffCreatePlaystack(
                    strptr,		/* Path. */
                    con_num,		/* Owner. */
		    atol(strv[1]),	/* YID. */
		    (strc >= 4) ? atoi(strv[3]) : 0,
                    (strc >= 5) ? atof(strv[4]) : 1.0,
                    (strc >= 6) ? atof(strv[5]) : 1.0,
                    (strc >= 7) ? atoi(strv[6]) : 1
                );
                /* YiffCreatePlaystack() will send response. */
	    }
	}
        /*
         *    Shutdown:    
         *
         *      YNET_CMD_SHUTDOWN
         */
        else if(!strcmp(strv[0], YNET_CMD_SHUTDOWN))  
        {
/* Need to check if connection has permission to do so. */
            YiffCloseConnection(con_num);
 
            runlevel = 1;
        }
        /*
         *    Sync:
         *
         *      YNET_CMD_SYNC cycle_ahead_us
         */
        else if(!strcmp(strv[0], YNET_CMD_SYNC))
        {
            if(recorder != NULL)
            {
                /* Recaliberate cycle if program calculates it. */
                if(recorder->audio.cycle_set == CYCLE_SET_PROGRAM)
                    YSoundCaliberateCycle(recorder);

                /* Sync sound device. */
                YSoundSync(recorder, 0);
            
                /* Calculate cycle ahead. */
                if(strc > 1)
                {
                    /* User specified. */
                    cycle_ahead_ms = atol(strv[1]) / 1000;
                }
                else
                {
#ifdef __linux__
                    if(recorder->audio.allow_fragments)
                    {
                        cycle_ahead_ms = 1.5 *
                            recorder->audio.compensated_cycle.ms;
                    }
                }
#endif  /* __linux__ */
            }

	    sprintf(sndbuf, "%s\n", YNET_CMD_SYNC);
	    CON_NOTIFY(con_num, sndbuf);
        }
        /*
         *    System stats:
         *
         *      YNET_CMD_SYSSTATS
	 *
	 *    Output to client looks like:
	 *
	 *	YNET_CMD_SYSSTATS cycle_us comp_cycle_us
	 *	fragments buffer_len channels sample_size sample_rate
	 *	start_time cycle_load
         */
        else if(!strcmp(strv[0], YNET_CMD_SYSSTATS))
        {
	    if(recorder != NULL)
	    {
	        sprintf(sndbuf, "%s %ld %ld %i %i %i %i %i %ld %.4lf\n",
                    YNET_CMD_SYSSTATS,
		    (recorder->audio.cycle.ms * 1000) +
                        recorder->audio.cycle.us,
                    (recorder->audio.compensated_cycle.ms * 1000) +
                        recorder->audio.compensated_cycle.us,
#ifdef SOUND_EXPERIMENTAL
		    recorder->audio.num_fragments,
#else
		    1,
#endif  /* SOUND_EXPERIMENTAL */
                    recorder->sound.buffer_length,
                    recorder->audio.channels,
                    recorder->audio.sample_size,
                    recorder->audio.sample_rate,
		    stats.start_time,
		    stats.cycle_load
		);
	    }
	    else
	    {
                sprintf(sndbuf, "%s %i %i %i %i %i %i %i %i %.4lf\n",
		    YNET_CMD_SYSSTATS,
		    0, 0, 0, 0, 0, 0, 0, 0, 0.00
		);
	    }
	    CON_NOTIFY(con_num, sndbuf);
        }

	StringFreeArray(strv, strc);


	return;
}


/*
 *	Check for and handle any incoming connections.
 */
int YiffCheckNewConnection(int socket)
{
	int i;
	YConnection *con_ptr;

        struct timeval timeout;
        fd_set readfds;
        int new_socket;

        int sin_size;
        struct sockaddr_in foreign_addr;
	char sndbuf[256];


        timeout.tv_sec = 0;
        timeout.tv_usec = 0;
        FD_ZERO(&readfds);
        FD_SET(socket, &readfds);
        select(socket + 1, &readfds, NULL, NULL, &timeout);
  
        if(!FD_ISSET(socket, &readfds))
            return(0);

        /* Handle new connection. */
        sin_size = sizeof(struct sockaddr_in);
        new_socket = accept(
            socket,
            (struct sockaddr *)&foreign_addr,
            &sin_size
        );
        if(new_socket == -1)
            return(-1);
	/* Set new socket non-blocking. */
        fcntl(new_socket, F_SETFL, O_NONBLOCK);


	/* Allocate a new connection structure. */
	i = ConnectionAllocate();
	con_ptr = ConnectionGetPtr(i);
	if(con_ptr == NULL)
	{
	    close(new_socket);
	    new_socket = -1;

	    return(-1);
	}	


	/* Set up connection. */
	con_ptr->socket = new_socket;
	con_ptr->host = StringCopyAlloc(inet_ntoa(foreign_addr.sin_addr));

	/* Is incoming connection's host allowed? */
	if(!YHostInList(con_ptr->host))
	{
	    sprintf(sndbuf,
 "%s %s %s Your host address `%s' is not authorized to connect.\n",
		YNET_CMD_DISCONNECT,
		YIFF_NAME,
		YIFF_VERSION,
                con_ptr->host
	    );
	    CON_NOTIFY(i, sndbuf);

	    ConnectionDelete(i);
	}


	return(0);
}


void YiffCloseConnection(YConnectionNumber con_num)
{
	int i;
	YConnection *con_ptr;
	char sndbuf[512];


	con_ptr = ConnectionGetPtr(con_num);
	if(con_ptr == NULL)
	    return;


	/* Notify connection of disconnect. */
	sprintf(sndbuf, "%s\n",
	    YNET_CMD_DISCONNECT
	);
	CON_NOTIFY(con_num, sndbuf);


	/* Delete all playstacks associated with this connection. */
	for(i = 0; i < total_playstacks; i++)
	{
	    if(playstack[i] == NULL)
		continue;

	    if(playstack[i]->owner == con_num)
	    {
		/* This playstack is owned by connection, delete it. */
		PlayStackDelete(i);

if(option.debug)
    fprintf(stderr,
        "Playstack %i deleted due to owner disconnecting.\n", i
    );
	    }
	}


	/* Close network connection. */
	if(con_ptr->socket > -1)
	{
	    close(con_ptr->socket);
	    con_ptr->socket = -1;
	}


	/*   Delete connection structure, freeing all it's substructures
         *   and allocated resources.
         */
	ConnectionDelete(con_num);


	return;
}


void YiffManageConnection(YConnectionNumber con_num)
{
        struct timeval timeout;
        fd_set readfds;
        int i, socket;
	char buf[NET_BUFFER_MAX];
	int bytes_read;

	int strc;
	char **strv;

	YConnection *con_ptr;


	con_ptr = ConnectionGetPtr(con_num);
        if(con_ptr == NULL)
            return;
        if(con_ptr->socket < 0)
	    return;


	/* Does connection have data to be read? */
	socket = con_ptr->socket;
        timeout.tv_sec = 0;
        timeout.tv_usec = 0;
        FD_ZERO(&readfds);
        FD_SET(socket, &readfds);
        select(socket + 1, &readfds, NULL, NULL, &timeout);
        if(!FD_ISSET(socket, &readfds))
            return;

	/* Read data. */
	bytes_read = recv(socket, buf, NET_BUFFER_MAX, 0);
	if(bytes_read <= 0)
	    return;

	if(bytes_read > NET_BUFFER_MAX)
	    bytes_read = NET_BUFFER_MAX;


	/* Null terminate. */
	buf[bytes_read - 1] = '\0';
	CRSanitize(buf);


	/* ******************************************************** */
	/* Begin parsing. */
	strv = strchrexp(buf, '\n', &strc);
	if((strv == NULL) ||
           (strc <= 0)
	)
	    return;

	/* Do each command. */
	for(i = 0; i < strc; i++)
	{
	    YiffDoConnectionCommand(con_num, strv[i]);
	}

	StringFreeArray(strv, strc);


	return;
}


void YiffManageConnections()
{
	int i;


        /*   Check for and handle any new incoming
	 *   connections.
	 */
	YiffCheckNewConnection(listen_socket);


        /* Handle each connected connection. */
        for(i = 0; i < total_yconnections; i++)
            YiffManageConnection(i);


	return;
}



/*
 *	Attempts to allocate a new playstack, if successful,
 *	then sets it up to the given values and loads it according
 *	to the file specified in path.
 *
 *	owner is the connection number that is to own this
 *	playstack.
 */
int YiffCreatePlaystack(
	char *path,
	YConnectionNumber owner,
	YID yid,
	YDataPosition pos,
	Coefficient volume_left,
	Coefficient volume_right,
	int repeats
)
{
	int i;
	PlayStack *ps_ptr;
	int bytes_per_cycle;
	char sndbuf[1024];


	if(!ConnectionIsConnected(owner))
	    return(-1);

	if((recorder == NULL) ||
           (path == NULL) ||
	   (yid == YIDNULL)
	)
	{
	    sprintf(sndbuf,
		"%s %i\n",
		YNET_CMD_PLAY,
		YIDNULL
	    );
            CON_NOTIFY(owner, sndbuf);

	    return(-1);
	}

	if(volume_left > 1)
	    volume_left = 1;
	if(volume_left < 0)
	    volume_left = 0;

        if(volume_right > 1)
            volume_right = 1;
        if(volume_right < 0)
            volume_right = 0;


	/* Allocate new playstack. */
	i = PlayStackAllocate();
	ps_ptr = PlayStackGetPtr(i);
	if(ps_ptr == NULL)
	{
            sprintf(sndbuf,
                "%s %i\n",
                YNET_CMD_PLAY,
                YIDNULL
            );
            CON_NOTIFY(owner, sndbuf);

	    return(-1);
	}

	ps_ptr->yid = yid;
	ps_ptr->owner = owner;
	ps_ptr->position = pos;
	ps_ptr->repeats = 0;
        ps_ptr->total_repeats = repeats;
	ps_ptr->data_length = 0;

	ps_ptr->volume_left = volume_left;
	ps_ptr->volume_right = volume_right;

	/* Get bytes per cycle. */
        bytes_per_cycle = recorder->sound.buffer_length;

	/* Open audio file. */
	if(
	    AFWOpen(
	        path,
	        &ps_ptr->afw_data
	    )
	)
	{
	    PlayStackDelete(i);

            sprintf(sndbuf,
                "%s %i\n",
                YNET_CMD_PLAY,
                YIDNULL
            );
            CON_NOTIFY(owner, sndbuf);

	    return(-1);
	}

	ps_ptr->data_length = ps_ptr->afw_data.entire_length;

	ps_ptr->block_align = ps_ptr->afw_data.block_align;
	ps_ptr->block_length = ps_ptr->afw_data.block_length;


	/* Load first segment. */
        AFWLoadSegment(
            &ps_ptr->afw_data,
            ps_ptr->position,
	    bytes_per_cycle
        );

	/* Notify connection of successful create. */
        sprintf(sndbuf,
            "%s %i\n",
            YNET_CMD_PLAY,
            ps_ptr->yid
        );
        CON_NOTIFY(owner, sndbuf);



	return(0);
}


void YiffManageSound()
{
	int i;
	PlayStack *ps_ptr;
	int bytes_per_cycle;

	SoundBuffer *tar_buf, *src_buf;
	YDataLength len;

	char sndbuf[1024];


	if(recorder == NULL)
	    return;

	if(1)
        {
	    /* Record or play? */
	    if(recorder->audio.direction == AUDIO_DIRECTION_RECORD)
	    {
		/* Record. */

	    }
	    else	/* AUDIO_DIRECTION_PLAY */
	    {
		/* Play. */

	        /* Play and reset midway buffer. */
	        YSoundPlayBuffer(recorder);

	        bytes_per_cycle = recorder->sound.buffer_length;
	        tar_buf = recorder->sound.buffer;

	        /* Manage each playstack. */
	        for(i = 0; i < total_playstacks; i++)
	        {
		    ps_ptr = playstack[i];

		    if(ps_ptr == NULL)
		        continue;


		    /* ********************************************* */
		    /* Mix to buffer. */
                    src_buf = ps_ptr->afw_data.buffer;
		    if(src_buf != NULL)
		    {
                        len = MIN(bytes_per_cycle,
                                  ps_ptr->afw_data.length
                        );      


		        YiffMixBuffers(
			    &recorder->audio,
		            tar_buf,
			    src_buf,
			    len,
			    ps_ptr->volume_left,
                            ps_ptr->volume_right,
                            ps_ptr->volume_up,
                            ps_ptr->volume_down,
                            ps_ptr->volume_away
		        );
		    }


		    /* ********************************************** */
		    /* Increment position. */
		    ps_ptr->position += bytes_per_cycle;

		    if(ps_ptr->position >= ps_ptr->data_length)
		    {
		        ps_ptr->position = 0;

		        /* Increment repeats. */
		        ps_ptr->repeats++;

		        /* Repeats exceeded? */
		        if((ps_ptr->total_repeats >= 0) &&
                           (ps_ptr->repeats >= ps_ptr->total_repeats)
		        )
		        {
			    sprintf(sndbuf, "%s %i\n",
				YNET_CMD_KILL,
				ps_ptr->yid
			    );
			    CON_NOTIFY(ps_ptr->owner, sndbuf);

			    PlayStackDelete(i);

			    continue;
		        }
		    }
		    /* Read next buffer segment. */
		    if(
		        AFWLoadSegment(
		            &ps_ptr->afw_data,
		            ps_ptr->position,
			    bytes_per_cycle
		        )
		    )
		    {
                        /* Error loading audio data segment. */

                        sprintf(sndbuf, "%s %i\n",
                            YNET_CMD_KILL,
                            ps_ptr->yid
                        );
                        CON_NOTIFY(ps_ptr->owner, sndbuf);

                        PlayStackDelete(i);

                        continue;
		    }
	        }


                /* Shift midway buffer. */
	        for(i = 0; i < bytes_per_cycle; i++)
	            tar_buf[i] = (int)tar_buf[i] + (int)128;
	    }
        }


        return;
}


/*
 *	Procedure to update timmers.
 */
void YiffUpdateTimmers()
{
	YTime new_millitime;
	time_t audio_cycle_us;


	GetCurrentTime(&new_millitime);
	if(new_millitime.ms < cur_time.ms)
	{
	    /*   Timmers need to be reset. Note that cur_time will
	     *   be updated in YiffResetTimmers();
	     */
	    YiffResetTimmers();
	}
	else
	{
	    if(recorder != NULL)
	    {
	        /* Get audio cycle in microseconds. */
		audio_cycle_us = (recorder->audio.compensated_cycle.ms *
                    1000) + recorder->audio.compensated_cycle.us;

                /*
                 *   Calculate next_sound_delta_us, this is the
		 *   delta time from now untill the next sound
		 *   sound be played.
		 *
		 *   Audio device cycle - (current time -
		 *   time audio was last played) - cycle ahead
		 *   - 100 us
		 */
	        next_sound_delta_us = audio_cycle_us -
                    (((new_millitime.ms * 1000) + new_millitime.us) -
                    ((cur_time.ms * 1000) + cur_time.us)) -
                    (cycle_ahead_ms * 1000)
                    - 100;

		/* Sanitize. */
		if(next_sound_delta_us > audio_cycle_us)
		    next_sound_delta_us = audio_cycle_us;
		if(next_sound_delta_us < 10)
		    next_sound_delta_us = 10;

		/* Stats: calculate cycle_load. */
		if(audio_cycle_us != 0)
		    stats.cycle_load = (double)(audio_cycle_us -
		        next_sound_delta_us) /
		        (double)audio_cycle_us;

                /* Decrease cycle_ahead_ms as needed. */
                cycle_ahead_ms = MAX(cycle_ahead_ms -
                    recorder->audio.compensated_cycle.ms,
                    0
                );

if(option.debug)
{
    if(cycle_ahead_ms > 0)
        fprintf(stderr,
            "cycle_ahead_ms reduced to %ld ms\n",
            cycle_ahead_ms
        );
}

/*
fprintf(stderr, "===> %ld (%ld %ld) us\n",
 next_sound_delta_us,
 recorder->audio.compensated_cycle.ms,
(((new_millitime.ms * 1000) + new_millitime.us) -
                    ((cur_time.ms * 1000) + cur_time.us))
);
*/

	    }
	    else
	    {
		/*   No audio device connected, set a reasonable
		 *   value for next_sound_delta_us since it is
		 *   used for the main loop usleep()
		 */
		next_sound_delta_us = 8000;

                /* Status: calculate cycle_load. */
                stats.cycle_load = 0.00;

                /* Decrease needed cycle_ahead_ms. */
                cycle_ahead_ms = 0;
	    }


	    /* Update cur_time as usual. */
	    cur_time.ms = new_millitime.ms;
	    cur_time.us = new_millitime.us;
	}


	return;
}


void YiffResetTimmers()
{
	/* Directly get current time (for syncing audio device). */
	GetCurrentTime(&cur_time);

	/* Reset next schedual timmers. */
	next.refresh.ms = 0;
	next.refresh.us = 0;


        /* Sync audio device. */
        YSoundSync(recorder, 0);

        /* Calculate cycle_ahead_ms. */
        if(recorder != NULL)
        {
#ifdef __linux__
            cycle_ahead_ms = 1.5 *
		recorder->audio.compensated_cycle.ms;
#else
	    cycle_ahead_ms = recorder->audio.compensated_cycle.ms;
#endif  /* __linux__ */
	}

	/* How long to sleep before playing sound again. */
	next_sound_delta_us = 10;	/* Use minimum. */

if(option.debug)
    fprintf(stderr, "Timming reset.\n");


	return;
}



int main(int argc, char *argv[])
{
	int status;


	runlevel = 1;

	/* Update all timmers. */
	YiffUpdateTimmers();

	/* Initialize everything. */
	status = YiffInit(argc, argv);
	switch(status)
	{
	  case -4:
	    YiffShutdown();
	    return(0);
	    break;

	  case 0:
	    break;

	  default:
	    YiffShutdown();
	    return(1);
	    break;
	}

	/* Main loop. */
	runlevel = 2;
	while(runlevel > 1)
	{
	    GetCurrentTime(&cur_time);	/* Need new time just after sleep. */

            YiffManageSound();
            YiffManageConnections();

	    /* Refresh? */
	    if(next.refresh.ms < cur_time.ms)
	    {
		if(recorder != NULL)
		{
		    YSoundSync(recorder, 0);
#ifdef __linux__
		    cycle_ahead_ms = 1.5 *
                        recorder->audio.compensated_cycle.ms;
#else
		    cycle_ahead_ms = recorder->audio.compensated_cycle.ms;
#endif	/* __linux__ */

if(option.debug)
    fprintf(stderr, "---------------[ Refresh ]-------------\n");
		}
		next.refresh.ms = cur_time.ms + option.refresh_int.ms;
	    }

            YiffUpdateTimmers();  

	    usleep(next_sound_delta_us);
	}

	/* Shutdown everything. */
	YiffShutdown();

	runlevel = 0;

	return(0);
}
