/*
                            YIFF Sound Server
 */

#ifndef YIFF_H
#define YIFF_H

#include <stdio.h>
#include <db.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>

#include "ytypes.h"
#include "ytimming.h"

#include "soundpaths.h"
#include "yconnection.h"
#include "ysound.h"
#include "playstack.h"

#include "afw.h"


/*
 *	Program name and version:
 */
#define YIFF_NAME	"YIFF"
#define YIFF_VERSION	"1.00"

#define YIFF_VERSION_MAJOR	1
#define YIFF_VERSION_MINOR	0


/*
 *	Program help message:
 */
#define YIFF_HELP "\
Usage: yiff [rcfile] [options]\n\
\n\
    [rcfile] specifies the restart configuration file to use.\n\
\n\
    [options] can be any of the following:\n\
\n\
        --device <device>             Specifies the sound device.\n\
        --port <port>                 Specifies the port to handle\n\
                                      incoming connections on.\n\
        --path <path>                 Adds a sound path (can be used\n\
                                      more than once).\n\
\n"


/*
 *	Default paths:
 */
#define DEF_RCFILE_NAME		"/usr/etc/yiffrc"


/*
 *	Listening Socket Backlog:
 */
#define LISTEN_SOCKET_BACKLOG   10

/*
 *	Network buffer size per connection:
 *
 *	Mainly used for reciving data from connections.
 */
#define NET_BUFFER_MAX		10000


/*
 *	Options:
 */
struct {

	Boolean		debug;

	int port;

	int sample_size;	/* 8 or 16. */
	int channels;		/* 1 or 2. */
	int sample_rate;

#ifdef SOUND_EXPERIMENTAL
	Boolean allow_fragments;
        u_int8_t num_fragments;
        u_int8_t fragment_size;
#endif  /* SOUND_EXPERIMENTAL */

	Boolean flip_stereo;
	int direction;		/* One of AUDIO_DIRECTION_* */

	int cycle_set;		/* One of CYCLE_SET_* */
        YDeltaTime cycle;	/*   User set cycle (if cycle_set is
                                 *   CYCLE_SET_USER).
                                 */

	YDeltaTime refresh_int;

} option;


/*
 *	Program statistics:
 */
struct {

	/*
	 *    When the program was started (in systime seconds):
	 */
	time_t start_time;

	/*    Cycle load in microseconds. Calculated by taking the
         *    compensated cycle interval - the time it took to
         *    process stuff per loop / the compensated cycle interval.
	 *
	 *    Values from 0.0 to 0.4 are very safe, values from
	 *    0.4 to 0.8 means moderate load, and values from
         *    0.8 to 0.99 means the the cpu cannot keep up with
	 *    the processing demand.
	 */
	Coefficient cycle_load;

} stats;

/*
 *	File names:
 */
struct {

	char rcfile[PATH_MAX + NAME_MAX];

	char device[PATH_MAX + NAME_MAX];

} fname;

/*
 *	Program master run level:
 */
int runlevel;


/*
 *	Current time since midnight:
 */
YTime cur_time;



/*
 *	Next operation timmers:
 *
 *	(In milliseconds unless noted otherwise.)
 */
struct {

	YTime	refresh;

} next;


/*
 *	Sound device data pointer.
 */
Recorder *recorder;

/*
 *	Network incoming connections listening socket:
 */
int listen_socket;




/* *************************************************************** */

/* In bufmix.c */
extern void YiffMixBuffers(
	Audio *audio,
        SoundBuffer *tar_buf,
        SoundBuffer *src_buf,
        YDataLength len,
        Coefficient volume_left,
        Coefficient volume_right,
        Coefficient volume_up,
        Coefficient volume_down,
        Coefficient volume_away
);


/* In yiff.c */
extern void YiffResetMixer();
extern void YiffSignalHandler(int sig);

extern int YiffInit(int argc, char **argv);
extern void YiffShutdown();

extern void YiffDoConnectionCommand(
	YConnectionNumber con_num,
	char *cmd
);
extern void YiffCloseConnection(YConnectionNumber con_num);
extern int YiffCheckNewConnection(int socket);
extern void YiffManageConnection(YConnectionNumber con_num);
extern void YiffManageConnections();

extern int YiffCreatePlaystack(
	char *path,
	YConnectionNumber owner,
        YID yid,
        YDataPosition pos,
        Coefficient volume_left,
        Coefficient volume_right,
	int repeats
);
extern void YiffManageSound();

extern void YiffUpdateTimmers();
extern void YiffResetTimmers();




#endif	/* YIFF_H */
