/*
	Audio preset modes.

	Used in simple mode changing by referancing a mode preset by
	an identifier name.
 */

#ifndef YMODES_H
#define YMODES_H

#include "ytimming.h"
#include "ytypes.h"


typedef struct {

	/*   Identifier name (alias) to refferance this mode
	 *   by.
	 */
	char *name;

	/* Audio parameters, see structure Audio in ysound.h. */
        YDeltaTime      cycle;
        YDeltaTime      write_ahead;
        int sample_size;
        int channels;
        int sample_rate;
        YDataLength bytes_per_second;	/* (not used). */
#ifdef SOUND_EXPERIMENTAL
        Boolean allow_fragments;
        u_int8_t num_fragments;
        u_int8_t fragment_size;	/* OS Native units, not bytes! */
#endif  /* SOUND_EXPERIMENTAL */
        Boolean flip_stereo;
        int direction;

} YMode;

YMode **ymode;
int total_ymodes;


extern int YModeIsAllocated(int n);
extern YMode *YModeGetPtr(int n);
extern int YModeMatch(char *name);

extern int YModeAllocate(char *name);
extern void YModeReset(int n);
extern void YModeDelete(int n);

extern void YModeDeleteAll();
extern void YModeReclaim();


#endif	/* YMODES_H */
