#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include "../include/yclientio.h"
#include "yiffconfig.h"


int ModeIsAllocated(int n)
{
        if((n < 0) ||
           (n >= total_modes) ||
           (mode == NULL)
        )
            return(0);
        else if(mode[n] == NULL)
            return(0);
        else
            return(1);
}


mode_struct *ModeGetPtr(int n)
{
        if(ModeIsAllocated(n))
            return(mode[n]);
        else
            return(NULL);
}


/*
 *	Attempts to match a mode by name, returns the
 *	mode_struct index number of -1 on no match.
 */
int ModeMatch(char *name)
{
	int i;


	if(name == NULL)
	    return(-1);


	for(i = 0; i < total_modes; i++)
	{
	    if(mode[i] == NULL)
		continue;

	    if(mode[i]->name == NULL)
		continue;


	    if(!strcmp(mode[i]->name, name))
		return(i);
	}


	return(-1);
}


/*
 *	Allocates a new mode_struct, returns -1 on allocation
 *	error or the mode_struct index on success.
 */
int ModeAllocate(char *name)
{
        int i, n, len;

 
        for(i = 0; i < total_modes; i++)
        {   
            if(mode[i] == NULL)
                break;
        }
        if(i < total_modes)
        {
            n = i;
        }
        else
        {
            n = total_modes;
            total_modes++;

            mode = (mode_struct **)realloc(
                mode,
                total_modes * sizeof(mode_struct *)
            );
            if(mode == NULL)
            {
                total_modes = 0;
                return(-1);
            }
        }

        mode[n] = (mode_struct *)calloc(1,
            sizeof(mode_struct)
        );
        if(mode[n] == NULL)
        {
            return(-1);
        }
 

	/* ************************************************************ */
        /* Set values. */

	ModeReset(n);

	/* Set name. */
	if(name != NULL)
	{
	    len = strlen(name);
	    mode[n]->name = (char *)calloc(1, (len + 1) * sizeof(char));
	    if(mode[n]->name != NULL)
		strcpy(mode[n]->name, name);
	}


        return(n);
}


/*
 *      Resets mode values to default, freeing any allocated
 *      substructures.
 */
void ModeReset(int n)
{
        mode_struct *ptr;

        ptr = ModeGetPtr(n);
        if(ptr == NULL)  
            return;

	/* Free name. */
	free(ptr->name);
	ptr->name = NULL;

	ptr->cycle_us = 1000000;

	ptr->write_ahead_us = 0;

        ptr->sample_size = 8;
        ptr->channels = 1;
        ptr->sample_rate = 11025;

        ptr->allow_fragmenting = True;
        ptr->num_fragments = 0x02;
        ptr->fragment_size = 0x0a;

        ptr->flip_stereo = False;
        ptr->direction = 0;


        return;
}


void ModeDelete(int n)
{
        if(ModeIsAllocated(n))  
        {
            /* Free allocated substructures. */
            ModeReset(n);

            free(mode[n]);
            mode[n] = NULL;
        }

        return;
}


void ModeDeleteAll()
{
        int i;

        for(i = 0; i < total_modes; i++)
        {
            ModeDelete(i);
        }

        free(mode);
        mode = NULL;

        total_modes = 0;

        return;
}



void ModeReclaim()
{




	return;
}
