/*
                  Restart Configuration File

	Functions:

	int RCLoadFromFile(char *filename)
	int RCSaveToFile(char *filename)

	---

 */

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <sys/stat.h>

#include "../include/cfgfmt.h"
#include "../include/string.h"
#include "../include/strexp.h"
#include "../include/fio.h"

#include "../include/yclientio.h"
#include "yiffconfig.h"



#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))



extern int antishift(int in);


int RCLoadFromFile(char *filename)
{
	int status;
        char *strptr, *strptr2;

        FILE *fp;
        off_t filesize;
        struct stat stat_buf;

        char parm[CFG_PARAMETER_MAX];
        char val[CFG_VALUE_MAX];
        int lines_read = 0;

	mode_struct *mode_ptr;
	sound_path_struct *sp_ptr;


	/* Check if filename exists. */
	if(filename == NULL)
	    return(-1);
	if(stat(filename, &stat_buf))
	{
	    fprintf(stderr, "%s: No such file.\n", filename);
	    return(-1);
	}

	/* Get size of file. */
        filesize = stat_buf.st_size;

        /* Open filename. */
        fp = fopen(filename, "r");
        if(fp == NULL)
        {
            fprintf(stderr, "%s: Cannot open.\n", filename);
            return(-1);
        }


	/* ************************************************************* */
	/* Free any allocated resources as needed. */

	ModeDeleteAll();
	SoundPathDeleteAll();


	/* ******************************************************** */

	strptr = NULL;

        while(1)
        {
	    /* Free previous line and allocate/read next line. */
            free(strptr); strptr = NULL;
            strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
            if(strptr == NULL) break;
            lines_read++;

            /* Fetch parameter. */
            strptr2 = StringCfgParseParm(strptr);
            if(strptr2 == NULL) continue;
            strncpy(parm, strptr2, CFG_PARAMETER_MAX);
            parm[CFG_PARAMETER_MAX - 1] = '\0';

            /* Fetch value. */
            strptr2 = StringCfgParseValue(strptr);
            if(strptr2 == NULL) strptr2 = "0";  /* Set it to "0" if NULL. */
            strncpy(val, strptr2, CFG_VALUE_MAX);
            val[CFG_VALUE_MAX - 1] = '\0';


	    /* VersionMajor */
            if(!strcmp(parm, "VersionMajor"))
            {

	    }
            /* VersionMinor */
            else if(!strcmp(parm, "VersionMinor"))
            {

            }

	    /* Port */
            else if(!strcmp(parm, "Port"))
            {
		option.port = atoi(val);
	    }
	    /* RefreshInterval */
            else if(!strcmp(parm, "RefreshInterval"))
            {
		option.refresh_int_us = atol(val);
            }
	    /* Device */
	    else if(!strcmp(parm, "Device"))
	    {
		strncpy(
		    option.device,
		    val,
		    PATH_MAX + NAME_MAX
		);
                if(stat(option.device, &stat_buf))
                {
                    fprintf(stderr,
   "%s: Line %i: Warning: No such device `%s'.\n",
                            filename,
                            lines_read,
                            option.device
                    );
                }
	    }

            /* **************************************************** */
	    /* BeginMode */
            else if(!strcmp(parm, "BeginYMode") ||
                    !strcmp(parm, "BeginMode")
	    )
            {
		status = ModeAllocate(NULL);
		mode_ptr = ModeGetPtr(status);
		if(mode_ptr == NULL)
                    continue;

                while(1)
                {
                    /* Free previous line and allocate/read next line. */
                    free(strptr); strptr = NULL;
                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
                    if(strptr == NULL) break;
                    lines_read++;
                    
                    /* Fetch parameter. */
                    strptr2 = StringCfgParseParm(strptr);
                    if(strptr2 == NULL) continue;
                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
                    parm[CFG_PARAMETER_MAX - 1] = '\0';
                        
                    /* Fetch value. */
                    strptr2 = StringCfgParseValue(strptr);  
                    if(strptr2 == NULL) strptr2 = "0";
                    strncpy(val, strptr2, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';


                    /* Name */
                    if(!strcmp(parm, "Name"))
                    {
			free(mode_ptr->name);
			mode_ptr->name = StringCopyAlloc(val);
		    }
                    /* Cycle */
                    else if(!strcmp(parm, "Cycle"))
                    {
                        mode_ptr->cycle_us = atol(val);
                    }
                    /* WriteAhead */
                    else if(!strcmp(parm, "WriteAhead"))
                    {
                        mode_ptr->write_ahead_us = atol(val);
                    }
                    /* SampleSize */
                    else if(!strcmp(parm, "SampleSize"))
                    {
                        mode_ptr->sample_size = atoi(val);

			if(mode_ptr->sample_size == 16)
			    mode_ptr->sample_size = 16;
			else
			    mode_ptr->sample_size = 8;
                    }
                    /* Channels */
                    else if(!strcmp(parm, "Channels"))
                    {
                        mode_ptr->channels = atoi(val);

                        if(mode_ptr->channels == 2)
                            mode_ptr->channels = 2;
                        else
                            mode_ptr->channels = 1;
                    }
                    /* SampleRate */
                    else if(!strcmp(parm, "SampleRate"))
                    {
                        mode_ptr->sample_rate = atoi(val);
                    }
                    /* AllowFragmenting */
                    else if(!strcmp(parm, "AllowFragmenting"))
                    {
                        mode_ptr->allow_fragmenting = StringIsYes(val);
                    }
                    /* Fragments */
                    else if(!strcmp(parm, "Fragments"))
                    {
                        mode_ptr->num_fragments = atoi(val);
                    }
                    /* FragmentSize */
                    else if(!strcmp(parm, "FragmentSize"))
                    {
			/* On file it's in bytes, we want it in shifts. */
                        mode_ptr->fragment_size = antishift(atoi(val) - 1);
                    }
                    /* FlipStereo */
                    else if(!strcmp(parm, "FlipStereo"))
                    {
                        mode_ptr->flip_stereo = StringIsYes(val);
                    }
                    /* Direction */
                    else if(!strcmp(parm, "Direction"))
                    {
			if(!strcmp(val, "Record") ||
                           !strcmp(val, "record")
			)
			    mode_ptr->direction = 1;
			else
                            mode_ptr->direction = 0;
                    }

                    /* EndMode */
                    else if(!strcmp(parm, "EndYMode") ||
                            !strcmp(parm, "EndMode")
		    )
                    {
                        break;
                    }

                    /* Unknown parameter. */
                    else
                    { 
                        fprintf(stderr,
                            "%s: Line %i: Unknown parameter: %s\n",
                            filename,
                            lines_read,
                            parm
                        );
                        continue;
                    }

                }
            }

            /* **************************************************** */
            /* BeginSoundPath */
            else if(!strcmp(parm, "BeginSoundPath"))
            {
                status = SoundPathAllocate();
                sp_ptr = SoundPathGetPtr(status);
                if(sp_ptr == NULL)
                    continue;

                while(1)
                {
                    /* Free previous line and allocate/read next line. */
                    free(strptr); strptr = NULL;
                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
                    if(strptr == NULL) break;
                    lines_read++;

                    /* Fetch parameter. */
                    strptr2 = StringCfgParseParm(strptr);
                    if(strptr2 == NULL) continue;
                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
                    parm[CFG_PARAMETER_MAX - 1] = '\0';

                    /* Fetch value. */
                    strptr2 = StringCfgParseValue(strptr);
                    if(strptr2 == NULL) strptr2 = "0";
                    strncpy(val, strptr2, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';  


                    /* Path */   
                    if(!strcmp(parm, "Path"))
                    {
                        free(sp_ptr->path);
                        sp_ptr->path = StringCopyAlloc(val);
                    }
                    /* EndSoundPath */
                    else if(!strcmp(parm, "EndSoundPath"))
                    {
                        break;
                    }
                    /* Unknown parameter. */  
                    else
                    {
                        fprintf(stderr,
                            "%s: Line %i: Unknown parameter: %s\n",
                            filename,
                            lines_read,
                            parm
                        );
                        continue;
                    } 
                }
            }

            /* Unknown parameter. */
            else
            {
                fprintf(stderr, "%s: Line %i: Unknown parameter: %s\n",
                    filename,
                    lines_read,
                    parm
                );
            }
	}


	/* Close the file. */
	fclose(fp); fp = NULL;


	return(0);
}


/*
 *      Saves configuration to filename.  Returns 0 on success
 *      and -1 on error.
 */
#define WRITE   bytes_written += fwrite(buf,\
                                        sizeof(char),\
                                        strlen(buf),\
                                        fp\
                );

int RCSaveToFile(char *filename)
{
        int i;
	char *strptr;
        char buf[CFG_STRING_MAX];
        FILE *fp;

	mode_struct *mode_ptr;
	list_window_struct *lw;
	page_stepper_struct *ps;

        int bytes_written = 0;



	if(filename == NULL)
	    return(-1);

	ps = &yc_win.ps;
	if(ps == NULL)
	    return(-1);

        fp = fopen(filename, "w");
        if(fp == NULL)
        {
            fprintf(stderr,
                "%s: Cannot open for writing.\n",
                filename
            );
            return(-1);
        }


        /* ********************************************************** */
        /* Header. */
        sprintf(buf,
            "# ##################################################################\n"
        );
        WRITE

        sprintf(buf,
            "#\n"
        );
        WRITE

        sprintf(buf,
            "# YIFF Sound Server: Configuration File\n"
	);
	WRITE

        sprintf(buf,
            "#\n\n"
        );
        WRITE


	/* Configuration file version. */
        sprintf(buf,
            "# Configuration file version:\n# \n"
        );
        WRITE
        sprintf(buf,
            "VersionMajor = %i\n",
	    PROG_VERSION_MAJOR
        );
        WRITE
        sprintf(buf,
            "VersionMinor = %i\n",
            PROG_VERSION_MINOR
        );
        WRITE

        sprintf(buf,
            "\n\n"
        );
        WRITE


        /* Listening port number. */
        sprintf(buf,
            "# Listening port number:\n# \n"
        );
        WRITE
        sprintf(buf,
            "Port = %i\n",
            option.port
        );
        WRITE

        sprintf(buf,
            "\n\n"    
        );
        WRITE


        /* Refresh interval (in microseconds). */
        sprintf(buf,
            "# Refresh interval (in microseconds):\n# \n"
        );
        WRITE
        sprintf(buf,
            "RefreshInterval = %ld\n",
            option.refresh_int_us
        );
        WRITE

        sprintf(buf,
            "\n\n"  
        );
        WRITE


        /* Sound device. */
        sprintf(buf,
            "# Sound device:\n# \n"
        );
        WRITE
        sprintf(buf,
            "Device = %s\n",
            option.device
        );
        WRITE

        sprintf(buf,
            "\n\n"
        );  
        WRITE


	/* ******************************************************** */
	/* Modes list. */
        sprintf(buf,
"# ##################################################################\n"
	);
	WRITE

	sprintf(buf,
"#\n"
	);
	WRITE

        sprintf(buf,
"# Modes list:\n"
	);
        WRITE

        sprintf(buf,
"#\n"
	);
        WRITE

        sprintf(buf,
"#   Notes:\n"
	);
        WRITE

        sprintf(buf,
"#\n"
	);
        WRITE

        sprintf(buf,
"#   All timming values are in microseconds.\n"
	);
        WRITE

        sprintf(buf,
"#\n"
	);
        WRITE

        sprintf(buf,
"#   FragmentSize values are device specific!\n"
	);
        WRITE

        sprintf(buf,
"#\n\n"
	);
        WRITE


	lw = yc_win.mode_list;
	if(lw != NULL)
	{
	    for(i = 0; i < lw->total_entries; i++)
	    {
		if(lw->entry[i] == NULL)
		    continue;

		/* Get mode pointer. */
		mode_ptr = (mode_struct *)lw->entry[i]->data_ptr;
		if(mode_ptr == NULL)
		    continue;


		if(i == 0)
		{
		    /* Mode #0. */
                    sprintf(buf,
"# Default mode, this must be defined and should be the first entry.\n"
	            );
                    WRITE

                    sprintf(buf,
"# YIFF will start up running in this mode.\n"
	            );
                    WRITE

                    sprintf(buf,
"#\n"
	            );
                    WRITE
		}

                sprintf(buf, "BeginMode\n");
	        WRITE

                sprintf(buf, "    Name = %s\n",
		    lw->entry[i]->name
		);
                WRITE

                sprintf(buf, "    Cycle = %ld\n",
		    mode_ptr->cycle_us
		);
                WRITE

                sprintf(buf, "    WriteAhead = %ld\n",
                    mode_ptr->write_ahead_us
                );
                WRITE

                sprintf(buf, "    SampleSize = %i\n",
                    mode_ptr->sample_size
		);
                WRITE

                sprintf(buf, "    Channels = %i\n",
                    mode_ptr->channels
                );
                WRITE

                sprintf(buf, "    SampleRate = %i\n",
                    mode_ptr->sample_rate
                );
                WRITE

                sprintf(buf, "    AllowFragmenting = %s\n",
		    (mode_ptr->allow_fragmenting) ? "yes" : "no"
		);
                WRITE

		sprintf(buf, "    Fragments = %i\n",
		    mode_ptr->num_fragments
		);
                WRITE

		/* In memory, it's shifts, on file we want it in bytes. */
		sprintf(buf, "    FragmentSize = %i\n",
		    (1 << mode_ptr->fragment_size)
		);
                WRITE

                sprintf(buf, "    FlipStereo = %s\n", 
                    (mode_ptr->flip_stereo) ? "yes" : "no"
                );
                WRITE

                sprintf(buf, "    Direction = %s\n",
                    (mode_ptr->direction == 1) ? "Record" : "Play"
                );
                WRITE

                sprintf(buf, "EndMode\n\n");
                WRITE
	    }
	}


        /* ******************************************************** */
        /* Modes list. */
        sprintf(buf,
"# ##################################################################\n"
        );
        WRITE

        sprintf(buf,
"#\n"
	);
        WRITE

        sprintf(buf,
"# Sound paths:\n"
	);
        WRITE

        sprintf(buf,
"#\n"
	);
        WRITE

        sprintf(buf,
"#   Notes:\n"
	);
        WRITE

        sprintf(buf,
"#\n"
	);
        WRITE

        sprintf(buf,
"#   You may specify as many paths as you want.\n"
	);
        WRITE

        sprintf(buf,
"#\n\n"
	);
        WRITE


	/* Path 1. */
	strptr = (char *)PStepperGetWidgetValuePtr(
		ps,
		WIDN_SOUND_PATH1_PROMPT
	);
	if(strptr != NULL)
	{
	    if(strptr[0] != '\0')
	    {
	        sprintf(buf, "BeginSoundPath\n");
                WRITE
	        sprintf(buf, "    Path = %s\n",
		    strptr
	        );
                WRITE
	        sprintf(buf, "EndSoundPath\n\n");
                WRITE
	    }
	}

        /* Path 2. */
        strptr = (char *)PStepperGetWidgetValuePtr(
                ps,
                WIDN_SOUND_PATH2_PROMPT
        );
        if(strptr != NULL)
        {
            if(strptr[0] != '\0')
            {
                sprintf(buf, "BeginSoundPath\n");
                WRITE
                sprintf(buf, "    Path = %s\n",
                    strptr
                );
                WRITE
                sprintf(buf, "EndSoundPath\n\n");
                WRITE
            }
        }

        /* Path 3. */
        strptr = (char *)PStepperGetWidgetValuePtr(
                ps,
                WIDN_SOUND_PATH3_PROMPT
        );
        if(strptr != NULL)
        {
            if(strptr[0] != '\0')
            {
                sprintf(buf, "BeginSoundPath\n");
                WRITE
                sprintf(buf, "    Path = %s\n",
                    strptr
                );
                WRITE
                sprintf(buf, "EndSoundPath\n\n");
                WRITE
            }
        }

        /* Path 4. */
        strptr = (char *)PStepperGetWidgetValuePtr(
                ps,
                WIDN_SOUND_PATH4_PROMPT
        );
        if(strptr != NULL)
        {
            if(strptr[0] != '\0')
            {
                sprintf(buf, "BeginSoundPath\n");
                WRITE
                sprintf(buf, "    Path = %s\n",
                    strptr
                );
                WRITE
                sprintf(buf, "EndSoundPath\n\n");
                WRITE
            }
        }

        /* Path 5. */
        strptr = (char *)PStepperGetWidgetValuePtr(
                ps,
                WIDN_SOUND_PATH5_PROMPT
        );
        if(strptr != NULL)
        {
            if(strptr[0] != '\0')
            {
                sprintf(buf, "BeginSoundPath\n");
                WRITE
                sprintf(buf, "    Path = %s\n",
                    strptr
                );
                WRITE
                sprintf(buf, "EndSoundPath\n\n");
                WRITE
            }
        }

        /* Path 6. */
        strptr = (char *)PStepperGetWidgetValuePtr(
                ps,
                WIDN_SOUND_PATH6_PROMPT
        );
        if(strptr != NULL)
        {
            if(strptr[0] != '\0')
            {
                sprintf(buf, "BeginSoundPath\n");
                WRITE
                sprintf(buf, "    Path = %s\n",
                    strptr
                );
                WRITE
                sprintf(buf, "EndSoundPath\n\n");
                WRITE
            }
        }

        /* Path 7. */
        strptr = (char *)PStepperGetWidgetValuePtr(
                ps,
                WIDN_SOUND_PATH7_PROMPT
        );
        if(strptr != NULL)
        {
            if(strptr[0] != '\0')
            {
                sprintf(buf, "BeginSoundPath\n");
                WRITE
                sprintf(buf, "    Path = %s\n",
                    strptr
                );
                WRITE
                sprintf(buf, "EndSoundPath\n\n");
                WRITE
            }
        }

        /* Path 8. */
        strptr = (char *)PStepperGetWidgetValuePtr(
                ps,
                WIDN_SOUND_PATH8_PROMPT
        );
        if(strptr != NULL)
        {
            if(strptr[0] != '\0')
            {
                sprintf(buf, "BeginSoundPath\n");
		WRITE
                sprintf(buf, "    Path = %s\n",
                    strptr
                );
		WRITE
                sprintf(buf, "EndSoundPath\n\n");
		WRITE
            }
        }





	return(0);
}
