/*
                          Widget: Toggle Button

	Functions:

	int TgBtnInit(
	        toggle_button_struct *tb,
	        win_t parent,
	        int x, int y,
	        bool_t state,
	        char *label
	)
	int TgBtnSetHintMessage(
		toggle_button_struct *tb,
		win_t parent,
		int x, int y,
		char *message
	)
	int TgBtnDraw(toggle_button_struct *tb, int amount)
	int TgBtnManage(toggle_button_struct *tb, event_t *xevent)
	void TgBtnMap(toggle_button_struct *tb)
	void TgBtnUnmap(toggle_button_struct *tb)
	void TgBtnDestroy(toggle_button_struct *tb)

 */


#include "../include/widget.h"


/* Minimum sizes for the toggle button. */
#define TB_MIN_WIDTH	6	/* Margins * 2 + min size 2. */
#define TB_MIN_HEIGHT	6

/* Margins. */
#define TB_MARGIN	2	/* In pixels. */

/* Sizes of characters (in pixels). */
#define TB_CHAR_WIDTH	7
#define TB_CHAR_HEIGHT	14



int TgBtnInit(
	toggle_button_struct *tb,
	win_t parent,
	int x, int y,
	bool_t state,
	char *label
)
{
	int len;
	image_t *ximage;


	if(!IDC() ||
           (tb == NULL) ||
	   (parent == 0) ||
	   (osw_gui[0].root_win == 0)
	)
	    return(-1);



	/* ************************************************************** */

	bzero(tb, sizeof(toggle_button_struct));

	tb->map_state = 0;
	tb->visibility_state = VisibilityFullyObscured;
	tb->x = x;
	tb->y = y;
	tb->is_in_focus = 0;
	tb->disabled = False;
	tb->font = OSWQueryCurrentFont();

	/* Calculate width and height. */
	ximage = widget_global.toggle_btn_unarmed_img;
	len = (label == NULL) ? 0 : strlen(label);
	tb->width = ((ximage == NULL) ? 16 : ximage->width) +
            (int)(TB_CHAR_WIDTH * len) + (int)(TB_MARGIN * 3);
	tb->height = ((ximage == NULL) ? 16 : ximage->height) +
	    (int)(TB_MARGIN * 2);


	/* Set toggle button state. */
	tb->state = state;

	/* Allocate label? */
	if(label != NULL)
	{
	    /* Allocate and copy label. */
	    len = strlen(label);
	    tb->label = (char *)calloc(1, (len + 1) * sizeof(char) );
	    if(tb->label == NULL)
		return(-1);

	    strcpy(tb->label, label);
	}
	else
	{
	    tb->label = NULL;
	}


	/* ************************************************************* */

	/* Initialize toplevel. */
        if(
	    OSWCreateWindow(
                &tb->toplevel,
	        parent,
	        tb->x, tb->y,
	        tb->width, tb->height
	    )
	)
	    return(-1);

        OSWSetWindowInput(
	    tb->toplevel,
	    ButtonPressMask | ButtonReleaseMask |
	    ExposureMask | VisibilityChangeMask |
            EnterWindowMask | LeaveWindowMask
        );


	/* Add widget to Regeristry. */
	WidgetRegAdd((void *)tb, WTYPE_CODE_TOGGLEBTN);



	return(0);
}



/*
 *	Set hint message for toggle button.   If parent is 0,
 *	then the parent of the toggle button is used.
 */
int TgBtnSetHintMessage(
	toggle_button_struct *tb,
	win_t parent,
	int x, int y,
	char *message
)
{
        int i;


        if(!IDC() ||
           (tb == NULL) ||
           (message == NULL) 
        )
            return(-1);


        /* Must be initialized. */
        if(tb->toplevel == 0)
	{
	    fprintf(stderr,
		"TgBtnSetHintMessage(): Structure not initialized.\n"
	    );
            return(-1);
	}

        /* If parent is 0, use structure's parent. */
        if(parent == 0)
	{
	    parent = OSWGetWindowParent(tb->toplevel);
	    if(parent == 0)
	       return(-1);
	}


        i = HintWinAddMessage(
            tb->toplevel, 
            parent,
            x, y,
            message
        );

        if(i < 0)
            return(-1);

	return(0);
}


int TgBtnDraw(toggle_button_struct *tb, int amount)
{
	int x, y;
	image_t *ximage;
	win_attr_t wattr;
        font_t *prev_font;


	if(!IDC() ||
           (tb == NULL)
	)
	    return(-1);


	/* Do not draw if mapped but fully obscured. */
	if((tb->map_state == 1) &&
           (tb->visibility_state == VisibilityFullyObscured)
	)
	    return(0);


	/* ********************************************************** */

	OSWGetWindowAttributes(tb->toplevel, &wattr);
        ximage = widget_global.toggle_btn_unarmed_img;

	/* Map as needed. */
	if(!tb->map_state)
	{
	    OSWMapRaised(tb->toplevel);

	    /* Must chage draw amount. */
	    amount = TGBTN_DRAW_AMOUNT_COMPLETE;

	    tb->map_state = 1;
	    tb->visibility_state = VisibilityUnobscured;

	    /* Recreate buffers. */
	    if(tb->toplevel_buf == 0)
	    {
		if(OSWCreatePixmap(
			&tb->toplevel_buf, wattr.width, wattr.height
	        ))
		    return(-1);
	    }
	}


        /* ********************************************************* */
         
        prev_font = OSWQueryCurrentFont();
        OSWSetFont(tb->font);

	if(1)
	{
	    /* Clear toplevel buffer. */
	    if(widget_global.force_mono)
	    {
	        OSWClearPixmap(
		    tb->toplevel_buf, wattr.width, wattr.height,
		    osw_gui[0].black_pix
		);
	    }
	    else
	    {
		WidgetPutImageTile(
		    tb->toplevel_buf, 
		    widget_global.std_bkg_img,
		    wattr.width, wattr.height
		);
	    }

	    /* Draw label. */
	    if(tb->label != NULL)
	    {
		x = ((ximage == NULL) ? 16 : ximage->width) +
                    (int)(TB_MARGIN * 2);
		y = ((int)wattr.height / 2) + 5;
		if(widget_global.force_mono)
		    OSWSetFgPix(osw_gui[0].white_pix);
		else if(tb->disabled)
		    OSWSetFgPix(widget_global.disabled_text_pix);
		else
                    OSWSetFgPix(widget_global.normal_text_pix);

		OSWDrawString(
		    (drawable_t)tb->toplevel_buf,
		    x, y,
		    tb->label
		);
	    }

	    /* Draw toggle button. */
            if(widget_global.force_mono)
            {
                OSWSetFgPix(osw_gui[0].white_pix);
                OSWDrawSolidRectangle(tb->toplevel_buf,
                    TB_MARGIN, TB_MARGIN, 16, 16);
                OSWSetFgPix(osw_gui[0].black_pix);
                OSWDrawRectangle(tb->toplevel_buf,
                    TB_MARGIN + 1, TB_MARGIN + 1, 13, 13);
                if(tb->state)
                {
                    OSWDrawLine(tb->toplevel_buf,
                        TB_MARGIN + 1, TB_MARGIN + 1,
                        TB_MARGIN + 14, TB_MARGIN + 14
                    );
                    OSWDrawLine(tb->toplevel_buf,
                        TB_MARGIN + 1, TB_MARGIN + 14,
                        TB_MARGIN + 14, TB_MARGIN + 1
                    );
                }
            }
	    else
            {   
                x = TB_MARGIN;
                y = TB_MARGIN;
                
                if(tb->state)
                {
                    WidgetPutImageNormal(
                        tb->toplevel_buf,
                        widget_global.toggle_btn_armed_img,
                        x, y,
                        True 
                    );
                }
                else
                {
                    WidgetPutImageNormal(
                        tb->toplevel_buf,
                        widget_global.toggle_btn_unarmed_img,
                        x, y,
                        True
                    );
                }
            }
	}

	OSWPutBufferToWindow(tb->toplevel, tb->toplevel_buf);

        OSWSetFont(prev_font);


	return(0);
}



int TgBtnManage(toggle_button_struct *tb, event_t *xevent)
{

	static int xevents_handled;


	if((tb == NULL) ||
           (xevent == NULL)
	)
	    return(0);

	/* Do not manage if not mapped. */
	if(tb->map_state == 0)
	    return(0);


	/* ****************************************************** */
	xevents_handled = 0;
	switch(xevent->type)
	{
          /* ********************************************************* */
          case ButtonPress:
	    if(xevent->xany.window == tb->toplevel)
	    {

		/* Always false if distabled. */
		if(tb->disabled)
		{
		    tb->state = False;
		}
		else if(tb->state)
		{
		    tb->state = False;
		}
                else
		{
                    tb->state = True;
		}

		/* Redraw. */
                TgBtnDraw(
                    (toggle_button_struct *)tb,
                    TGBTN_DRAW_AMOUNT_BUTTON
                );

		xevents_handled++;
		return(xevents_handled);
	    }
	    break;

          /* ********************************************************* */
	  case Expose:
	    if(xevent->xany.window == tb->toplevel)
	    {
		xevents_handled++;
	    }
	    break;

          /* ********************************************************* */
          case MotionNotify:
            break;

          /* ********************************************************* */
          case VisibilityNotify:
            if(xevent->xany.window == tb->toplevel)
            {
                tb->visibility_state = xevent->xvisibility.state;
                xevents_handled++;
                
                /* No need to continue, just return. */
                return(xevents_handled);
            }
            break;

          /* ********************************************************* */
          case EnterNotify:
            if(xevent->xany.window == tb->toplevel)
            {
                /* Schedual hints window map. */
                HintWinSetSchedual(
                    widget_global.hintwin_map_delay,
                    tb->toplevel
                );

                xevents_handled++;
                return(xevents_handled);
            }
            break;

          /* ********************************************************* */
          case LeaveNotify:  
            if(xevent->xany.window == tb->toplevel)
            {
                /* Unmap/unschedual hints window. */
                HintWinUnmap();

                xevents_handled++;
		return(xevents_handled);
            }
            break;
	}


	/* Redraw toggle button as needed. */
	if(xevents_handled > 0)
	{
	    TgBtnDraw(
		(toggle_button_struct *)tb,
		TGBTN_DRAW_AMOUNT_COMPLETE
	    );
	}


	return(xevents_handled);
}



void TgBtnMap(toggle_button_struct *tb)
{
        if(tb == NULL)
            return;


	tb->map_state = 0;
        TgBtnDraw(tb, TGBTN_DRAW_AMOUNT_COMPLETE);


	return;
}


void TgBtnUnmap(toggle_button_struct *tb)
{
	if(tb == NULL)
	    return;


	OSWUnmapWindow(tb->toplevel);
        tb->map_state = 0;
        tb->visibility_state = VisibilityFullyObscured;
        tb->is_in_focus = 0;

	/* Destroy large buffers. */
	OSWDestroyPixmap(&tb->toplevel_buf);


	return;
}



void TgBtnDestroy(toggle_button_struct *tb)
{
	if(tb == NULL) return;


        /* Delete widget from regeristry. */
        WidgetRegDelete((void *)tb);


	if(IDC())
	{
            /* Delete associated hint message. */
            HintWinDeleteMessage(tb->toplevel);

	    OSWDestroyWindow(&tb->toplevel);
	    OSWDestroyPixmap(&tb->toplevel_buf);
	}


	/* Free label. */
	free(tb->label); tb->label = NULL;
	tb->state = False;


        tb->map_state = 0; 
        tb->visibility_state = VisibilityFullyObscured;
	tb->is_in_focus = 0;
        tb->x = 0;
        tb->y = 0;
        tb->width = 0;
        tb->height = 0;
        tb->disabled = False;


	return;
}
