/*
                           YIFF Configurator

	Functions:

	int antishift(int in)

	int YCPageChangeCB(void *ptr, int prev_page, int new_page)
        int YCExitCB(void *ptr)
        int YCFinishCB(void *ptr)
	int YCModeChangeCB(void *ptr)
	int YCApplyChangesCB(void *ptr)
	int YCTestSoundCB(void *ptr)

	void YCUnfocusToplevels()

	int YCInit(int argc, char *argv[])
	void YCDraw()
	void YCManageEvents()
	void YCMap()
	void YCUnmap()
	void YCShutdown()

	int main(int argc, char *argv[])

	---

 */

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>


#include "../include/string.h"
#include "../include/strexp.h"

#include "../include/osw-x.h"
#include "../include/widget.h"
#include "../include/yclientio.h"

#include "yiffconfig.h"

/* Panel image. */
#include "yiff_panel.xpm"



/* 
 *      Anti shift macro:
 */
int antishift(int in)
{
        int s;
        int i = in;

        for(s = 0; i; i >>= 1)
            s++;

        return(s);
}



/*
 *	Page change callback function.
 */
int YCPageChangeCB(void *ptr, int prev_page, int new_page)
{
	char stringa[256];
	void *ptr2;
	char *con_arg;
	char *start_arg;

	page_stepper_struct *ps;


	if(ptr == NULL)
	    return(-1);

	ps = (page_stepper_struct *)ptr;


        /* ********************************************************* */
	/* Connect or disconnect from YIFF server? */
	if((prev_page == 0) &&
           (new_page == 1)
	)
	{
	    /* Connect to YIFF server. */
            ptr2 = PStepperGetWidgetValuePtr(ps, WIDN_YIFF_BINARY_PROMPT);
            start_arg = (char *)ptr2;
	    ptr2 = PStepperGetWidgetValuePtr(ps, WIDN_CONNECT_ADDRESS_PROMPT);
	    con_arg = (char *)ptr2;

	    yconnection = YIFFConnect(
		start_arg,
		con_arg
	    );
	    if(yconnection == NULL)
	    {
		fprintf(stderr, "Error connecting to YIFF server.\n");
	    }

	    /* Change to default mode. */
	    if(yc_win.mode_list != NULL)
	    {
		/* Force select mode 0. */
		yc_win.mode_list->entry_pos = 0;

		YCModeChangeCB(yc_win.mode_list);
	    }
	}
	else if((prev_page == 1) &&
                (new_page == 0)
        )
	{
	    /* Disconnect from YIFF server (but do not shut it down. */
	    YIFFDisconnect(yconnection, True);
	    yconnection = NULL;
	}




	/* ********************************************************* */
	/* Change page stepper title by page. */
	switch(new_page)
	{
	  case 0:
            sprintf(stringa, "%s: Connect", PROG_NAME);
	    break;

          case 1:
            sprintf(stringa, "%s: Mode Settings", PROG_NAME);
            break;

          case 2:
            sprintf(stringa, "%s: Sound Paths", PROG_NAME);
            break;

          case 3:
            sprintf(stringa, "%s: Save Configuration", PROG_NAME);
            break;
	}
        OSWSetWindowTitle(ps->toplevel, stringa);


	return(0);
}


/*
 *	Page stepper exit callback function.
 */
int YCExitCB(void *ptr)
{
        page_stepper_struct *ps;


        if(ptr == NULL)
            return(-1);

        ps = (page_stepper_struct *)ptr;




	runlevel = 1;


	return(0);
}


/*
 *	Page stepper finish callback function.
 */
int YCFinishCB(void *ptr)                             
{
	int status;
        page_stepper_struct *ps;
	char *strptr;
	char tmp_path[PATH_MAX + NAME_MAX];


        if(ptr == NULL)
            return(-1);

        ps = (page_stepper_struct *)ptr;
	if(ps == NULL)
	    return(-1);


	/* Get save to file name. */
	strptr = (char *)PStepperGetWidgetValuePtr(
            ps,
            WIDN_SAVE_TO_PROMPT
        );
	if(strptr != NULL)
	    strncpy(tmp_path, strptr, PATH_MAX + NAME_MAX);
	else
	    strncpy(tmp_path, fname.yiff_config, PATH_MAX + NAME_MAX);
	tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';


	/* Save configuration. */
	status = RCSaveToFile(tmp_path);
	if(status < 0)
	{
	    fprintf(stderr,
		"Error occured while saving configuration to `%s'\n",
		tmp_path
	    );
	}

	/* Switch to runlevel 1. */
	runlevel = 1;


	return(0);
}


/*
 *	Handler for when a new mode is selected (double clicked) on
 *	the mode list.
 */
int YCModeChangeCB(void *ptr)
{
	int i, n;
	int w_type;
	void *ptr2;

	list_window_struct *lw;
	mode_struct *mode_ptr;

	prompt_window_struct *prompt;
	toggle_button_array_struct *tba;


	if(ptr == NULL)
	    return(-1);

	/* Get pointer to mode list and selected entry position. */
	lw = yc_win.mode_list;
	n = lw->entry_pos;


	if(!ListIsItemAllocated(lw, n))
	    return(-1);


	/* Get pointer to mode on item. */
	for(i = 0; i < total_modes; i++)
	{
	    if(mode[i] == NULL)
		continue;

	    if((void *)mode[i] == lw->entry[n]->data_ptr)
		break;
	}
	if(i < total_modes)
	{
	    mode_ptr = mode[i];

	    /* Apply changes, new mode. */

            /* Sample rate. */
            if(!PStepperGetWidgetPointer(
                &yc_win.ps,
                WIDN_SAMPLE_RATE_PROMPT,
                &ptr2,
                &w_type
            ))
            {
                prompt = (prompt_window_struct *)ptr2;
                if(prompt->buf != NULL)
		    sprintf(prompt->buf, "%i", mode_ptr->sample_rate);
		PromptDraw(prompt, PROMPT_DRAW_AMOUNT_COMPLETE);
            }

            /* Sample size. */
            if(!PStepperGetWidgetPointer(
                &yc_win.ps,   
                WIDN_SAMPLE_SIZE_TBA,
                &ptr2,
                &w_type
            ))
            {
                tba = (toggle_button_array_struct *)ptr2;
		tba->armed_tb = (mode_ptr->sample_size == 16) ? 1 : 0;
                TgBtnArrayMap(tba);
            }

            /* Channels. */ 
            if(!PStepperGetWidgetPointer(
                &yc_win.ps,
                WIDN_CHANNELS_TBA,
                &ptr2,
                &w_type
            ))
            {
                tba = (toggle_button_array_struct *)ptr2;
                tba->armed_tb = (mode_ptr->channels == 2) ? 1 : 0;
                TgBtnArrayMap(tba);
            }

            /* Flip Stereo. */
/*
            if(!PStepperGetWidgetPointer(
                &yc_win.ps,
                WIDN_FLIP_STEREO_TB,
                &ptr2,
                &w_type
            ))
            { 
                tb = (toggle_button_struct *)ptr2;
                tb->state = (mode_ptr->flip_stereo) ? True : False;
                TgBtnDraw(tb);
            }
 */

            /* Fragments. */
            if(!PStepperGetWidgetPointer(
                &yc_win.ps,
                WIDN_FRAGMENTS_PROMPT,
                &ptr2,
                &w_type
            ))
            {
                prompt = (prompt_window_struct *)ptr2;
                if(prompt->buf != NULL)
                    sprintf(prompt->buf, "%i", mode_ptr->num_fragments);
                PromptDraw(prompt, PROMPT_DRAW_AMOUNT_COMPLETE);
            }

            /* Fragment size. */
            if(!PStepperGetWidgetPointer(
                &yc_win.ps,
                WIDN_FRAGMENT_SIZE_PROMPT,
                &ptr2,
                &w_type
            ))
            {
                prompt = (prompt_window_struct *)ptr2;
                if(prompt->buf != NULL)
                    sprintf(prompt->buf, "%i", 
			(int)(1 << mode_ptr->fragment_size)
		    );
                PromptDraw(prompt, PROMPT_DRAW_AMOUNT_COMPLETE);
            }

	    /* Cycle. */
	    yc_win.cycle_sb->pos = mode_ptr->cycle_us;
	    ScaleBarDraw(yc_win.cycle_sb);


	    /* Update mode to YIFF. */
            YIFFSetMode(  
                yconnection,
		mode_ptr->sample_size,
		mode_ptr->channels,
		mode_ptr->sample_rate,
		mode_ptr->direction,
		mode_ptr->allow_fragmenting,
		mode_ptr->num_fragments,
		(1 << mode_ptr->fragment_size)
            );

            YIFFSetCycle(
                yconnection,
                (long)yc_win.cycle_sb->pos
            );


	}



	return(0);
}


/*
 *	Add mode button callback function.
 */
int YCAddModeCB(void *ptr)
{
	int i, cur_pos;
	list_window_struct *lw;
	mode_struct *mode_ptr;


	lw = yc_win.mode_list;
	if(lw == NULL)
	    return(-1);


	/* Allocate new mode. */
	i = ModeAllocate("New Mode");
	mode_ptr = ModeGetPtr(i);
	if(mode_ptr == NULL)
	    return(-1);


	/* Allocate new item on list widget. */
	cur_pos = lw->entry_pos;

	i = ListAddItem(
            lw,
            LIST_ENTRY_TYPE_NORMAL,
            mode_ptr->name,
	    NULL,
	    cur_pos + 1,
            mode_ptr
        );
	if(i < 0)
	    return(-1);


	ListWinDraw(lw);


	return(0);
}


/*
 *	Delete mode button callback function.
 */
int YCDeleteModeCB(void *ptr)
{
        int i, cur_pos;
        list_window_struct *lw;
        mode_struct *mode_ptr;
  
 
        lw = yc_win.mode_list;
        if(lw == NULL)
            return(-1);


        /* Get mode_ptr. */
        cur_pos = lw->entry_pos;
        if(ListIsItemAllocated(lw, cur_pos))
	    mode_ptr = (mode_struct *)lw->entry[cur_pos]->data_ptr;
	else
	    return(-1);

	/* Delete mode. */
	if(mode_ptr != NULL)
	{
	    for(i = 0; i < total_modes; i++)
	    {
	        if(mode[i] == mode_ptr)
		    ModeDelete(i);
	    }
	}

	/* Delete list widget item. */
	ListDeleteItem(lw, cur_pos);

	ListWinDraw(lw);
        

        return(0);
}


/*
 *	Apply sound changes button callback function.
 */
int YCApplyChangesCB(void *ptr)
{
        int i, cur_pos;

	void *ptr2;

	long cycle;
        int sample_size;
        int channels;
        int sample_rate;
        int num_fragments;
        int fragment_size;

	list_window_struct *lw;
	page_stepper_struct *ps;
	mode_struct *mode_ptr;


	if(ptr == NULL)
	    return(-1);


	/* Get page stepper pointer. */
	ps = (page_stepper_struct *)&yc_win.ps;


	/* Get values from widgets. */
        if(yc_win.cycle_sb != NULL)
	    cycle = (long)yc_win.cycle_sb->pos;
	else
	    cycle = 25000;	/* Something is better than nothing. */

	sample_size =
	    ((int)PStepperGetWidgetValue(ps, WIDN_SAMPLE_SIZE_TBA) == 0) ?
                8 : 16;

	channels =
	    ((int)PStepperGetWidgetValue(ps, WIDN_CHANNELS_TBA) == 0) ?
                1 : 2;

        ptr2 = PStepperGetWidgetValuePtr(ps, WIDN_SAMPLE_RATE_PROMPT);
	if(ptr2 != NULL)
	    sample_rate = atoi((char *)ptr2);

        ptr2 = PStepperGetWidgetValuePtr(ps, WIDN_FRAGMENTS_PROMPT);
        if(ptr2 != NULL)
            num_fragments = atoi((char *)ptr2);

        ptr2 = PStepperGetWidgetValuePtr(ps, WIDN_FRAGMENT_SIZE_PROMPT);
        if(ptr2 != NULL)
            fragment_size = atoi((char *)ptr2);


	/* ********************************************************* */
	/* Apply changes to YIFF server. */
        if(yconnection != NULL)
        {
	    YIFFSetMode(
		yconnection,
		sample_size,
		channels,
		sample_rate,
		0,			/* direction */
		1,			/* allow_fragmenting */
		num_fragments,
		fragment_size		/* In bytes. */
	    );

            YIFFSetCycle(yconnection, cycle);
	}


        /* ********************************************************* */
	/* Apply changes to currently selected mode. */
	lw = yc_win.mode_list;
	if(lw == NULL)
	    return(-1);

	cur_pos = lw->entry_pos;
	if(!ListIsItemAllocated(lw, cur_pos))
	    return(0);

	for(i = 0; i < total_modes; i++)
	{
	    if(mode[i] == NULL)
		continue;

	    if((void *)mode[i] == lw->entry[cur_pos]->data_ptr)
	        break;
	}
	if(i < total_modes)
	{
	    mode_ptr = mode[i];

            mode_ptr->cycle_us = cycle;
	    /* Write ahead is calculated. */
	    mode_ptr->write_ahead_us = (double)cycle * 1.5;

	    mode_ptr->sample_rate = sample_rate;
	    mode_ptr->sample_size = sample_size;
	    mode_ptr->channels = channels;

	    mode_ptr->allow_fragmenting = 1;
	    mode_ptr->num_fragments = num_fragments;

	    /* Fragment size in mode must be in shifts. */
            mode_ptr->fragment_size = antishift(fragment_size - 1);

/* Flip stereo and direction? */

	}



	return(0);
}


/*
 *	Test sound callback.
 */
int YCTestSoundCB(void *ptr)
{
	char tmp_path[PATH_MAX + NAME_MAX];
        page_stepper_struct *ps;
	void *ptr2;


        if(ptr == NULL)
            return(-1);


        ps = (page_stepper_struct *)&yc_win.ps;

	if(yconnection != NULL)
	{
            ptr2 = PStepperGetWidgetValuePtr(ps, WIDN_TEST_SOUND_PROMPT);
            if(ptr2 != NULL)
	    {
		strncpy(tmp_path, (char *)ptr2, PATH_MAX + NAME_MAX);
		tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';


		YIFFDestroyPlay(yconnection, last_play_id);
		last_play_id = YIFFPlaySound(
		    yconnection,
		    tmp_path,
		    0,
		    1.0, 1.0,
		    1
		);
	    }
	}


	return(0);
}



/*
 *	Unfocuses all toplevel windows.
 */
void YCUnfocusToplevels()
{
	return;
}



/*
 *	Initializes the program.
 */
int YCInit(int argc, char *argv[])
{
	int i, page, status, w_type;
	void *ptr;

	page_stepper_struct *ps;

	char *strv[10];

	char stringa[512];


	yconnection = NULL;

	option.gui_con_arg = NULL;

	strcpy(fname.yiff_config, DEF_YIFF_CONFIG_NAME);
        strcpy(fname.yiff, DEF_YIFF_NAME);


	mode = NULL;
	total_modes = 0;


        /* *********************************************************** */
	/* Parse arguments. */
	for(i = 1; i < argc; i++)
	{
	    if(argv[i] == NULL)
		continue;

	    /* Help. */
	    if(strpfx(argv[i], "--h") ||
               strpfx(argv[i], "-h") ||
               strpfx(argv[i], "-?")
	    )
	    {
		printf(PROG_USAGE_MESG);
		return(-4);
	    }
	    /* Version. */
            else if(strpfx(argv[i], "--ver") ||
                    strpfx(argv[i], "-ver")
            )  
            {
                printf("%s Version %s\n",
		    PROG_NAME, PROG_VERSION
		);
                return(-4);
            }



	    /* RCFile (if all else fails. */
	    else
	    {
		strncpy(fname.yiff_config, argv[i], PATH_MAX + NAME_MAX);
		fname.yiff_config[PATH_MAX + NAME_MAX - 1] = '\0';
	    }
	}


	/* Load values from configuration file. */
	status = RCLoadFromFile(fname.yiff_config);
	if(status)
	{
	    fprintf(stderr,
		"Warning: Cannot find configuration file `%s'\n",
		fname.yiff_config
	    );
	}


        /* *********************************************************** */

        /* Connect to GUI. */
	status = OSWGUIConnect(argc, argv);
        if(status)
            return(-1);

        /* Initialize widget globals. */
	status = WidgetInitGlobals(argc, argv);
	if(status)
            return(-1);

	/* Uncomment this if you want to see how mono mode looks like. */
	/* widget_global.force_mono = True; */


	/* *********************************************************** */

	/* Load panel image. */
	yc_win.panel_img = WidgetLoadImageFromXpmData(yiff_panel);

	/* Get pointer to page stepper. */
	ps = &yc_win.ps;

        /* Initalize page stepper. */
	status = PStepperInit(
	    ps,
	    osw_gui[0].root_win,
	    0, 0,
	    600, 440,
	    4,				/* Total pages. */
	    yc_win.panel_img,
	    (void *)ps,			/* Client data. */
	    YCPageChangeCB,
            YCExitCB,
	    YCFinishCB
	);
	if(status)
	    return(-1);
	sprintf(stringa, "%s: Connect", PROG_NAME);
        OSWSetWindowTitle(ps->toplevel, stringa);


        /* *********************************************************** */
	/* Page #0. */
	page = 0;

	status = PStepperAllocateLabel(
	    ps,
	    page,
"Copyright 1997 1998 Taura (learfox@mailcity.com)\n\
Copyright 1999 Wolfpack Development Organization\n\
\n\
You are strongly encuraged to read\n\
http://www.yiff.org/yiff/config.htm\n\
to get the most complete information on configuring YIFF.\n\
\n\
Please enter the `address:port' to connect to the\n\
YIFF Sound Server.  Note that YIFF will automatically\n\
be started as needed.",
            NULL,
	    osw_gui[0].std_font,
	    widget_global.normal_text_pix,
	    142, 30
	);
        if(status)
            return(-1);

	/* Connect address prompt. */
	strv[0] = "Connect Argument:";
	strv[1] = "127.0.0.1:9433";
	strv[2] = "256";		/* buffer length. */
	status = PStepperAllocateWidget(
	    ps,
	    page,
	    WIDN_CONNECT_ADDRESS_PROMPT,
	    WTYPE_CODE_PROMPT,
            160, 220,
            300, YC_PROMPT_HEIGHT,
            3, strv
	);
        if(status)
            return(-1);

        /* Program path. */
        strv[0] = "YIFF Binary:";
        strv[1] = fname.yiff;
        strv[2] = "1024";		/* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_YIFF_BINARY_PROMPT,
            WTYPE_CODE_PROMPT,
            160, 250,
            300, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);


        /* *********************************************************** */
        /* Page #1. */
        page = 1;

        status = PStepperAllocateLabel(
            ps,   
            page,
	    "Modes List:",
            NULL,
            osw_gui[0].std_font,
            widget_global.normal_text_pix,
            160, 20
        );
        if(status)
            return(-1);

	/* Modes list. */
        status = PStepperAllocateWidget(
            ps,  
            page,
            WIDN_MODE_LIST, 
            WTYPE_CODE_LIST,
            157, 30,
            340, 105,
            1, strv
        );
        status = PStepperGetWidgetPointer(
            ps,
            WIDN_MODE_LIST,	/* Name. */
            &ptr,		/* Pointer return. */
            &w_type		/* Type. */
        );
        if(status)
            return(-1);  
        if(w_type == WTYPE_CODE_LIST)
            yc_win.mode_list = (list_window_struct *)ptr;
        else
            yc_win.mode_list = NULL;
	if(yc_win.mode_list != NULL)
	{
	    yc_win.mode_list->func_cb = YCModeChangeCB;
	    yc_win.mode_list->client_data = (void *)yc_win.mode_list;

	    for(i = 0; i < total_modes; i++)
	    {
	        if(mode[i] == NULL)
		    continue;

	        ListAddItem(
		    yc_win.mode_list,
		    LIST_ENTRY_TYPE_NORMAL,
		    mode[i]->name,
		    NULL,
		    -1,
		    (void *)mode[i]
	        );
	    }
	}


	/* Add mode button. */
        strv[0] = "Add";		/* Label name. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_ADD_MODE_BTN,
            WTYPE_CODE_PUSHBUTTON,
            510, 30,
            80, YC_BTN_HEIGHT,
            1, strv
        );
        if(status)
            return(-1);
        status = PStepperGetWidgetPointer(
            ps,
            WIDN_ADD_MODE_BTN,	/* Name. */
            &ptr,               /* Pointer return. */
            &w_type             /* Type. */
        );
        if(status)
            return(-1);
        if(w_type == WTYPE_CODE_PUSHBUTTON)
            yc_win.add_mode_btn = (push_button_struct *)ptr;
        else
            yc_win.add_mode_btn = NULL;
        if(yc_win.add_mode_btn != NULL)
        {
            yc_win.add_mode_btn->func_cb = YCAddModeCB;
        }


        /* Delete mode button. */
        strv[0] = "Delete";		/* Label name. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_DELETE_MODE_BTN,
            WTYPE_CODE_PUSHBUTTON,
            510, 70,
            80, YC_BTN_HEIGHT,
            1, strv
        );
        if(status)
            return(-1);
        status = PStepperGetWidgetPointer(
            ps,
            WIDN_DELETE_MODE_BTN,	/* Name. */
            &ptr,               /* Pointer return. */
            &w_type             /* Type. */
        );
        if(status)
            return(-1);
        if(w_type == WTYPE_CODE_PUSHBUTTON)
            yc_win.delete_mode_btn = (push_button_struct *)ptr;
        else
            yc_win.delete_mode_btn = NULL;
        if(yc_win.delete_mode_btn != NULL)
        {
            yc_win.delete_mode_btn->func_cb = YCDeleteModeCB;
        }


        /* Sound apply changes button. */
        strv[0] = "Apply Changes";      /* Label name. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SOUND_APPLY_BTN,
            WTYPE_CODE_PUSHBUTTON,
            470, 147,
            110, YC_BTN_HEIGHT,
            1, strv
        );
        if(status)
            return(-1);
        status = PStepperGetWidgetPointer(
            ps,
            WIDN_SOUND_APPLY_BTN,       /* Name. */
            &ptr,               /* Pointer return. */
            &w_type             /* Type. */
        );
        if(status)
            return(-1);
        if(w_type == WTYPE_CODE_PUSHBUTTON)
            yc_win.apply_sound_btn = (push_button_struct *)ptr;
        else
            yc_win.apply_sound_btn = NULL;
        if(yc_win.apply_sound_btn != NULL)
        {
            yc_win.apply_sound_btn->func_cb = YCApplyChangesCB;
        }

        /* Sample rate prompt. */
        strv[0] = "Sample Rate (in Hz):";
	sprintf(stringa, "%i", 8000);
        strv[1] = stringa;
        strv[2] = "40";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SAMPLE_RATE_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 145,
            230, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

	/* Sample size label. */
        status = PStepperAllocateLabel(
            ps,
            page,
            "Sample Size:",
            NULL,
            osw_gui[0].std_font,
            widget_global.normal_text_pix,
            160, 203
        );
        if(status)
            return(-1);
        /* Sample size toggle button array. */
        strv[0] = "8";
        strv[1] = "16";
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SAMPLE_SIZE_TBA,
            WTYPE_CODE_TOGGLEARRAY,
            255, 189,
            0, 0,
            2, strv
        );  
        if(status)
            return(-1);

        /* Channels label. */
        status = PStepperAllocateLabel(
            ps,
            page,
            "Channels:",
            NULL,
            osw_gui[0].std_font,
            widget_global.normal_text_pix,
            310, 203   
        );
        if(status)
            return(-1);
        /* Channels toggle button array. */
        strv[0] = "1";
        strv[1] = "2";
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_CHANNELS_TBA,
            WTYPE_CODE_TOGGLEARRAY,
            380, 189,
            0, 0,
            2, strv
        );
        if(status)
            return(-1);

        /* Fragments prompt. */
        strv[0] = "Fragments:";
        sprintf(stringa, "%i", 2);   
        strv[1] = stringa;
        strv[2] = "40";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_FRAGMENTS_PROMPT,  
            WTYPE_CODE_PROMPT,
            157, 235,
            130, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

        /* Fragment size prompt. */
        strv[0] = "Fragment Size (in Bytes):";
        sprintf(stringa, "%i", 512);
        strv[1] = stringa;
        strv[2] = "40";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_FRAGMENT_SIZE_PROMPT,
            WTYPE_CODE_PROMPT,
            310, 235,
            240, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

	/* Cycle interval scale bar. */
        status = PStepperAllocateLabel(
            ps,
            page,
            "Cycle Interval:",
            NULL,
            osw_gui[0].std_font,
            widget_global.normal_text_pix,
            160, 290
        );
        if(status)
            return(-1);
        strv[0] = "80000";              /* Cur. */
        strv[1] = "0";                  /* Min. */
        strv[2] = "250000";             /* Max. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_CYCLE_SB,
            WTYPE_CODE_SCALEBAR,   
            180, 300,
            400, 400,
            3, strv
        );
        if(status)
            return(-1);
        status = PStepperGetWidgetPointer(
            ps,
            WIDN_CYCLE_SB,      /* Name. */
            &ptr,               /* Pointer return. */
            &w_type             /* Type. */
        );
        if(status)
            return(-1);
        if(w_type == WTYPE_CODE_SCALEBAR)
            yc_win.cycle_sb = (scale_bar_struct *)ptr;
        else
            yc_win.cycle_sb = NULL;


        /* Test sound prompt. */
        strv[0] = "Test Sound:";
        strv[1] = "/tmp/file.wav";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_TEST_SOUND_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 340,
            340, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);
        status = PStepperGetWidgetPointer(
            ps,
            WIDN_TEST_SOUND_PROMPT,	/* Name. */
            &ptr,               /* Pointer return. */
            &w_type             /* Type. */
        );
        if(status)
            return(-1);
        if(w_type == WTYPE_CODE_PROMPT)
            yc_win.test_sound_prompt = (prompt_window_struct *)ptr;
        else
            yc_win.test_sound_prompt = NULL;


        /* Test sound button. */
        strv[0] = "Play";		/* Label name. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_TEST_SOUND_BTN,
            WTYPE_CODE_PUSHBUTTON,
            500, 340,
            70, YC_BTN_HEIGHT,
            1, strv
        );
        if(status)
            return(-1);
        status = PStepperGetWidgetPointer(
            ps,
            WIDN_TEST_SOUND_BTN,	/* Name. */
            &ptr,               /* Pointer return. */
            &w_type             /* Type. */
        );
        if(status)
            return(-1);
        if(w_type == WTYPE_CODE_PUSHBUTTON)
            yc_win.test_sound_btn = (push_button_struct *)ptr;
        else
            yc_win.test_sound_btn = NULL;
        if(yc_win.test_sound_btn != NULL)
        {
            yc_win.test_sound_btn->func_cb = YCTestSoundCB;
        }


        /* *********************************************************** */
        /* Page #2. */
        page = 2;

	/* Sound paths label. */
        status = PStepperAllocateLabel(
            ps,
            page,
"Sound search paths, non-absolute paths will be searched in these\n\
directories.",
            NULL,
            osw_gui[0].std_font,
            widget_global.normal_text_pix,  
            142, 30
        );
        if(status)
            return(-1);


        /* Sound path 1. */
        strv[0] = "Sound Path 1:";
        strv[1] = (total_soundpaths >= 1) ? soundpath[0]->path :
	    "";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SOUND_PATH1_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 60,
            400, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

        /* Sound path 2. */
        strv[0] = "Sound Path 2:";
        strv[1] = (total_soundpaths >= 2) ? soundpath[1]->path :
            "";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SOUND_PATH2_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 100,
            400, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

        /* Sound path 3. */
        strv[0] = "Sound Path 3:";
        strv[1] = (total_soundpaths >= 3) ? soundpath[2]->path :
            "";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SOUND_PATH3_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 140,
            400, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

        /* Sound path 4. */
        strv[0] = "Sound Path 4:";
        strv[1] = (total_soundpaths >= 4) ? soundpath[3]->path :
            "";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SOUND_PATH4_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 180,
            400, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

        /* Sound path 5. */
        strv[0] = "Sound Path 5:";
        strv[1] = (total_soundpaths >= 5) ? soundpath[4]->path :
            "";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SOUND_PATH5_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 220,
            400, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

        /* Sound path 6. */
        strv[0] = "Sound Path 6:";
        strv[1] = (total_soundpaths >= 6) ? soundpath[5]->path :
            "";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,  
            page,
            WIDN_SOUND_PATH6_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 260,
            400, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

        /* Sound path 7. */
        strv[0] = "Sound Path 7:";
        strv[1] = (total_soundpaths >= 7) ? soundpath[6]->path :
            "";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,  
            page,
            WIDN_SOUND_PATH7_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 300,
            400, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);

        /* Sound path 8. */
        strv[0] = "Sound Path 8:";
        strv[1] = (total_soundpaths >= 8) ? soundpath[7]->path :
            "";
        strv[2] = "1024";               /* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SOUND_PATH8_PROMPT,
            WTYPE_CODE_PROMPT,
            157, 340,
            400, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);




        /* *********************************************************** */
        /* Page #3. */
        page = 3;

        status = PStepperAllocateLabel(
            ps,
            page,
"You are now about to save your changes to disk.\n\
If you want to go back and modify any changes, you\n\
may press `back' otherwise press `finish' to\n\
save your changes.\n\n\
Note that if the YIFF server was not running prior to\n\
running this configuration program, it will be shut down.\n",
            NULL,
            osw_gui[0].std_font,
            widget_global.normal_text_pix,
            142, 30
        );
        if(status)
            return(-1);

        /* Test sound prompt. */
        strv[0] = "Save to:";		/* name label. */
        strv[1] = fname.yiff_config;	/* buffer contents. */
        strv[2] = "1024";		/* buffer length. */
        status = PStepperAllocateWidget(
            ps,
            page,
            WIDN_SAVE_TO_PROMPT,
            WTYPE_CODE_PROMPT,
            160, 220,
            300, YC_PROMPT_HEIGHT,
            3, strv
        );
        if(status)
            return(-1);



	return(0);
}


/*
 *	Redraws.
 */
void YCDraw()
{



	return;
}


/*
 *	Manages the events from the GUI and YIFF.
 */
void YCManageEvents()
{
	event_t event;
	YiffEvent yevent;
	int events_handled;	/* Events handled per loop. */


	/* ******************************************************** */
	/* Manage GUI events. */
        while(OSWEventsPending() > 0)
        {
	    /* Reset events counter. */
            events_handled = 0;

	    /* Get event. */
            OSWWaitNextEvent(&event);


            /*   Let WidgetManage() see this event.
	     *   It is not important if the event is handled or not, so
	     *   the return value is disgarded.
	     */
            WidgetManage(&event);


	    /* Enter key. */
	    if(event.type == KeyPress)
	    {
		if((event.xkey.keycode == osw_keycode.enter) ||
                   (event.xkey.keycode == osw_keycode.np_enter)
                )
		{
		    if(yc_win.test_sound_prompt != NULL)
		    {
			if(yc_win.test_sound_prompt->is_in_focus)
			{
			    YCTestSoundCB(yc_win.test_sound_btn);
			    continue;
			}
		    }
		}
	    }


	    /* Let our allocated widgets see this event too. */
	    events_handled += PStepperManage(
		&yc_win.ps,
		&event
	    );


	    /* Watch for event on cycle scalebar. */
	    if(yc_win.cycle_sb != NULL)
	    {
		if(event.xany.window == yc_win.cycle_sb->toplevel)
		{
		    /* Update cycle. */
		    if((event.type == ButtonPress) ||
                       ((event.type == MotionNotify) &&
                         yc_win.cycle_sb->btn_state
		       )
		    )
		    {
			if(yconnection != NULL)
			{
			    YIFFSetCycle(
				yconnection,
				(long)yc_win.cycle_sb->pos
			    );
			}
		    }
                    /* Sync on button release. */
                    if(event.type == ButtonRelease)
                    {
                        if(yconnection != NULL)
                        {
                            YIFFSync(
                                yconnection,
                                False
                            );
                        }
                    }
		}	/* Watch for event on cycle scalebar. */
	    }
        }

	/* ****************************************************** */
        /* Manage YIFF events. */

	while(1)
	{
	    events_handled = YIFFGetNextEvent(
	        yconnection,
	        &yevent,
	        False
            );
	    if(events_handled <= 0)
		break;

	    switch(yevent.type)
	    {
	      case YEventModeChange:
		break;

	      case YEventCycleChange:
		break;

	      case YEventDisconnect:
		YIFFDisconnect(yconnection, False);
		yconnection = NULL;
		break;

              case YEventHost:
                break;

	      case YEventSoundKill:
		break;

              case YEventMixer:
		break;

              case YEventSoundPlay:
 		break;

	      case YEventShutdown:
		break;

	      case YEventSync:
		break;

	      case YEventSystemStats:
		break;
	    }
	}


	return;
}


/*
 *	Map main windows.
 */
void YCMap()
{
	/* Map the widgets. */
	PStepperMap(&yc_win.ps);

	return;
}


/*
 *	Unmaps the main windows.
 */
void YCUnmap()
{
        /* Unmap the widgets. */
        PStepperUnmap(&yc_win.ps);

	return;
}


/*
 *	Deallocate all dynamic memory and resources for
 *	this program and the widget set.
 */
void YCShutdown()
{

	/* Disconnect from YIFF. */
	if(yconnection != NULL)
	{
            YIFFDisconnect(yconnection, False);
            yconnection = NULL;
	}


	/* Destroy page stepper. */
	PStepperDestroy(&yc_win.ps);

	/* Destroy panel image for page stepper. */
	OSWDestroyImage(&yc_win.panel_img);


	/*   Free widget globals after all client allocated widgets
	 *   have been deallocated.
	 */
        WidgetDestroyGlobals();

	/* Disconnect from the GUI. */
        OSWGUIDisconnect();

	free(option.gui_con_arg);
	option.gui_con_arg = NULL;


	/* Free all mode records. */
	ModeDeleteAll();

	/* Free all sound paths. */
	SoundPathDeleteAll();


	return;
}



int main(int argc, char *argv[])
{
	int status;


	runlevel = 1;

	status = YCInit(argc, argv);
	switch(status)
	{
	  case 0:
	    break;

	  case -4:
	    YCShutdown();
	    return(0);
	    break;

	  default:
            YCShutdown();
            return(0);
	    break;
	}

	YCMap();


	runlevel = 2;
	while(runlevel >= 2)
	{
	    usleep(8000);

	    /*   Must call WidgetManage() once per loop to let
	     *   the widget set do what it needs to do.
	     */
	    WidgetManage(NULL);

	    /* Manage events. */
	    YCManageEvents();
	}


	YCShutdown();

	runlevel = 0;


	return(0);
}
