#ifndef YIFFCONFIG_H
#define YIFFCONFIG_H


#include <stdio.h>

#include "../include/osw-x.h"
#include "../include/widget.h"


/*
 *	Program name and version:
 */
#define PROG_NAME	"YIFF Configurator"
#define PROG_VERSION	"1.00"

#define PROG_VERSION_MAJOR	1
#define PROG_VERSION_MINOR	0


/*
 *	Program usage message:
 */
#define PROG_USAGE_MESG	"\
Usage: yiffconfig [rcfile] [GUI_options]\n\
\n\
    [rcfile] specifies the restart configuration file to load.\n\
\n\
    [GUI_options] can be any options standard to your GUI, consult\n\
    your GUI's manual for available options.\n\
\n"



/*
 *	Default path locations:
 */
#define DEF_YIFF_CONFIG_NAME		"/usr/etc/yiffrc"

/*
 *	Default path locations:
 */
#define DEF_YIFF_NAME			"/usr/sbin/yiff"


/*
 *	Standard button sizes:
 */
#define YC_BTN_WIDTH	80
#define YC_BTN_HEIGHT	28

/*
 *	Standard prompt height:
 */
#define YC_PROMPT_HEIGHT	30


#ifndef PATH_MAX
# define PATH_MAX	1024
#endif

#ifndef NAME_MAX
# define NAME_MAX	256
#endif



/*
 *	Widget ID names:
 */
#define WIDN_YIFF_BINARY_PROMPT		"YIFF_BINARY_PROMPT"
#define WIDN_CONNECT_ADDRESS_PROMPT	"CONNECT_ADDRESS_PROMPT"

#define WIDN_MODE_LIST		"MODE_LIST"
#define WIDN_ADD_MODE_BTN	"ADD_MODE_BTN"
#define WIDN_DELETE_MODE_BTN	"DELETE_MODE_BTN"
#define WIDN_SOUND_APPLY_BTN	"SOUND_APPLY_BTN"
#define WIDN_SAMPLE_RATE_PROMPT	"SAMPLE_RATE_PROMPT"
#define WIDN_SAMPLE_SIZE_TBA 	"SAMPLE_SIZE_TBA"
#define WIDN_CHANNELS_TBA	"CHANNELS_TBA"
#define WIDN_FLIP_STEREO_TB	"FLIP_STEREO_TB"
#define WIDN_FRAGMENTS_PROMPT		"FRAGMENTS_PROMPT"
#define WIDN_FRAGMENT_SIZE_PROMPT	"FRAGMENT_SIZE_PROMPT"
#define WIDN_CYCLE_SB		"CYCLE_SB"
#define WIDN_TEST_SOUND_PROMPT		"TEST_SOUND_PROMPT"
#define WIDN_TEST_SOUND_BTN		"TEST_SOUND_BTN"

#define WIDN_SOUND_PATH1_PROMPT		"SOUND_PATH1_PROMPT"
#define WIDN_SOUND_PATH2_PROMPT         "SOUND_PATH2_PROMPT"
#define WIDN_SOUND_PATH3_PROMPT         "SOUND_PATH3_PROMPT"
#define WIDN_SOUND_PATH4_PROMPT         "SOUND_PATH4_PROMPT"
#define WIDN_SOUND_PATH5_PROMPT         "SOUND_PATH5_PROMPT"
#define WIDN_SOUND_PATH6_PROMPT         "SOUND_PATH6_PROMPT"
#define WIDN_SOUND_PATH7_PROMPT         "SOUND_PATH7_PROMPT"
#define WIDN_SOUND_PATH8_PROMPT         "SOUND_PATH8_PROMPT"

#define WIDN_SAVE_TO_PROMPT		"SAVE_TO_PROMPT"



/* *************************************************************** */

/*
 *	Program's run level:
 */
int runlevel;


/*
 *	Connection to YIFF server information:
 */
YiffConnection *yconnection;


/*
 *	YID of last play item (can be YIDNULL):
 */
YID last_play_id;

/*
 *	Options:
 */
struct {

	char *gui_con_arg;

	/* RC File options. */
	int port;
	long refresh_int_us;
	char device[PATH_MAX + NAME_MAX];

} option;


/*
 *	File names:
 */
struct {

	/* Configuration file for YIFF server currently being working on. */
	char yiff_config[PATH_MAX + NAME_MAX];

	/* YIFF server binary. */
	char yiff[PATH_MAX + NAME_MAX];

} fname;


/*
 *	YIFF Configurator window:
 */
struct {

	page_stepper_struct ps;

	image_t *panel_img;

	/* Widget pointers. */
	list_window_struct *mode_list;
	push_button_struct *add_mode_btn;
	push_button_struct *delete_mode_btn;
        push_button_struct *apply_sound_btn;
	scale_bar_struct *cycle_sb;
	prompt_window_struct *test_sound_prompt;
	push_button_struct *test_sound_btn;

} yc_win;


/*
 *	Mode records:
 */
typedef struct {

	char *name;

	long cycle_us;
	long write_ahead_us;

	int sample_rate;
	int sample_size;
	int channels;

	int allow_fragmenting;
	int num_fragments;
	int fragment_size;

	int flip_stereo;
	int direction;

} mode_struct;

mode_struct **mode;
int total_modes;


/*
 *	Sound paths:
 */
typedef struct {

	char *path;

} sound_path_struct;

sound_path_struct **soundpath;
int total_soundpaths;



/* **************************************************************** */

/* In mode.c */
extern int ModeIsAllocated(int n);
extern mode_struct *ModeGetPtr(int n);
extern int ModeMatch(char *name);
extern int ModeAllocate(char *name);
extern void ModeReset(int n);
extern void ModeDelete(int n);
extern void ModeDeleteAll();
extern void ModeReclaim();


/* In rcfile.c */
extern int RCLoadFromFile(char *filename);
extern int RCSaveToFile(char *filename);


/* In soundpaths.c */
extern int SoundPathIsAllocated(int n);
extern sound_path_struct *SoundPathGetPtr(int n);
extern char *SoundPathCompletePath(char *filename);
extern int SoundPathAllocate();
extern void SoundPathDelete(int n);
extern void SoundPathDeleteAll();


/* In yiffconfig.c */
extern int YCPageChangeCB(void *ptr, int prev_page, int new_page);
extern int YCExitCB(void *ptr);
extern int YCFinishCB(void *ptr);
extern int YCModeChangeCB(void *ptr);
extern int YCAddModeCB(void *ptr);
extern int YCDeleteModeCB(void *ptr);
extern int YCApplyChangesCB(void *ptr);
extern int YCTestSoundCB(void *ptr);

extern void YCUnfocusToplevels();
extern int YCInit(int argc, char *argv[]);
extern void YCDraw();
extern void YCManageEvents();
extern void YCMap();
extern void YCUnmap();
extern void YCShutdown();



#endif	/* YIFFCONFIG_H */
