#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include "../include/string.h"
#include "../include/yclientio.h"


#ifndef False
# define False	0
#endif

#ifndef True
# define True  1
#endif

#ifndef DEF_CON_ARG
# define DEF_CON_ARG    "127.0.0.1:9433"
#endif            



/*
 *	Print help.
 */
void PrintUsage()
{
	printf("\
Usage: yiffhost [-] <ip_address> [options]\n\
\n\
    Specifying a `-' character means to remove the <ip_address>.\n\
\n\
    <ip_address> should be a numeric IP address of the host you\n\
    want to allow access to the YIFF server.\n\
\n\
    [options] can be any of the following:\n\
\n\
        --recorder <address:port>    Specify which YIFF server to connect to.\n\
\n"
	);


	return;
}


int main(int argc, char *argv[])
{
	int i, status;
	char *strptr = NULL;

	char *address = NULL;
	char *con_arg = NULL;

	int op = 1;

	YiffConnection *con = NULL;
	YiffEvent event;


	if(argc < 2)
	{
	    PrintUsage();
	    return(1);
	}


	/* Parse arguments. */
	for(i = 1; i < argc; i++)
	{
	    if(argv[i] == NULL)
		continue;

	    /* Help. */
	    if(strpfx(argv[i], "--h") ||
               strpfx(argv[i], "-h")
	    )
	    {
		PrintUsage();
		return(0);
	    }

	    /* Add or remove? */
	    if(i == 1)
	    {
		if(!strcmp(argv[i], "-"))
		{
		    op = 0;	/* We know it's remove, continue. */
		    continue;
		}
		else if(!strcmp(argv[i], "+"))
		{
		    op = 1;	/* We know it's add, continue. */
                    continue;
		}
		else
		{
		    op = 1;
		}
	    }

	    /* Address. */
	    if((i == 1) ||
               (i == 2)
            )
	    {
		free(address);
		address = StringCopyAlloc(argv[i]);
		continue;
	    }

	    /* Connect address. */
	    if(strpfx(argv[i], "--rec") ||
               strpfx(argv[i], "-rec")
	    )
	    {
		i++;
		if(i < argc)
		{
		    free(con_arg);
		    con_arg = StringCopyAlloc(argv[i]);
		}
		else
		{
		    fprintf(stderr,
			"%s: Requires argument.\n",
			argv[i - 1]
		    );
		}
	    }
	}


	/*
	 *	Connect to YIFF server.
	 */
	con = YIFFConnect(
	    NULL,		/* No start argument. */
	    con_arg
	);
	if(con == NULL)
	{
	    printf("%s: Cannot connect to YIFF server.\n",
		(con_arg == NULL) ? DEF_CON_ARG : con_arg
	    );

	    free(address);
	    address = NULL;

            free(con_arg);
            con_arg = NULL;

	    return(-1);
	}


	/*
	 *	Add or remove host.
	 */
	status = YIFFHost(
            con, op, address
        );
	if(status)
	{
            printf("Cannot %s host `%s'.\n",
		(op == 0) ? "remove" : "add",
                address
            );
	}


	/*
	 *	Wait for response.
	 */
        YIFFGetNextEvent(
            con,
            &event,
	    True
        );
         


	/*
	 *	Disconnect from YIFF server
	 */
	YIFFDisconnect(con, False);
	con = NULL;

        free(address);
        address = NULL;

	free(con_arg);
	con_arg = NULL;



	return(0);
}
