#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include "../include/string.h"
#include "../include/yclientio.h"


#ifndef False
# define False	0
#endif

#ifndef True
# define True  1
#endif

#ifndef DEF_CON_ARG
# define DEF_CON_ARG    "127.0.0.1:9433"
#endif            



/*
 *	Parameter codes:
 */
#define PARM_CYCLE		1	/* Change cycle. */
#define PARM_MODE_PRESET	2	/* Change mode preset. */



/*
 *	Print help.
 */
void PrintUsage()
{
	printf("\
Usage: yiffset [<parameter> [value]] [options]\n\
\n\
    List of <parameter>s will be printed if no [<parameter> [value]]\n\
    is given\n\
\n\
    [options] can be any of the following:\n\
\n\
        --recorder <address:port>    Specify which YIFF server to connect to.\n\
\n"
	);


	return;
}


/*
 *	Print settings.
 */
void PrintSettings()
{

	printf("\
Available parameters:\n\n\
    --cycle <microseconds>       -- Adjusts the cycle.\n\
    --mode <preset_mode_name>    -- Changes the mode by preset name.\n\
\n"
	);


	return;
}


int main(int argc, char *argv[])
{
	int i, status;
	char *strptr = NULL;

	char *con_arg = NULL;

	int parm;
	char val[1024];

	YiffConnection *con = NULL;
	YiffEvent event;


	if(argc < 2)
	{
	    PrintSettings();
	    return(0);
	}


	/* Parse arguments. */
	for(i = 1; i < argc; i++)
	{
	    if(argv[i] == NULL)
		continue;

	    /* Help. */
	    if(strpfx(argv[i], "--h") ||
               strpfx(argv[i], "-h")
	    )
	    {
		PrintUsage();
		return(0);
	    }

	    /* Cycle. */
	    else if(strpfx(argv[i], "--c") ||
                    strpfx(argv[i], "-c")
	    )
	    {
                i++;
                if(i < argc)
                {
                    parm = PARM_CYCLE;
		    strncpy(val, argv[i], 1024);
		    val[1024 - 1] = '\0';
                }   
                else
                {
                    fprintf(stderr,
                        "%s: Requires argument.\n",
                        argv[i - 1]
                    );
                }
	    }
            /* Mode (preset mode name). */
            else if(strpfx(argv[i], "--m") ||
                    strpfx(argv[i], "-m")
            )
            {
                i++;
                if(i < argc)
                {
                    parm = PARM_MODE_PRESET;
                    strncpy(val, argv[i], 1024);
                    val[1024 - 1] = '\0';
                }
                else
                {
                    fprintf(stderr,
                        "%s: Requires argument.\n",
                        argv[i - 1]
                    );
                }
            }


	    /* Connect address. */
	    else if(strpfx(argv[i], "--rec") ||
                    strpfx(argv[i], "-rec")
	    )
	    {
		i++;
		if(i < argc)
		{
		    free(con_arg);
		    con_arg = StringCopyAlloc(argv[i]);
		}
		else
		{
		    fprintf(stderr,
			"%s: Requires argument.\n",
			argv[i - 1]
		    );
		}
	    }
	}


	/*
	 *	Connect to YIFF server.
	 */
	con = YIFFConnect(
	    NULL,		/* No start argument. */
	    con_arg
	);
	if(con == NULL)
	{
	    printf("%s: Cannot connect to YIFF server.\n",
		(con_arg == NULL) ? DEF_CON_ARG : con_arg
	    );

            free(con_arg);
            con_arg = NULL;

	    return(-1);
	}


	/*
	 *	Apply value change.
	 */
	status = -1;
	switch(parm)
	{
	  case PARM_CYCLE:
            status = YIFFSetCycle(
		con,
		atol(val)
	    );
            break;

	  case PARM_MODE_PRESET:
            status = YIFFSetModePreset(
                con,
		val
            );
	    break;
	}
	if(status)
	{
	    fprintf(stderr,
                "Could not set value `%s'.\n",
		val
	    );
	}


        /*
         *      Wait for response.
         */
        YIFFGetNextEvent(
            con,
            &event,
            True
        );  



	/*
	 *	Disconnect from YIFF server
	 */
	YIFFDisconnect(con, False);
	con = NULL;

	free(con_arg);
	con_arg = NULL;


	return(0);
}
