#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <unistd.h>

#include "../include/string.h"
#include "../include/yclientio.h"


#ifndef False
# define False	0
#endif

#ifndef True
# define True  1
#endif

#ifndef DEF_CON_ARG
# define DEF_CON_ARG    "127.0.0.1:9433"
#endif            

#ifndef MAX_PROTOCOL_STATEMENT
# define MAX_PROTOCOL_STATEMENT		8000
#endif


/*
 *	Parameter codes:
 */
#define PARM_CYCLE		1	/* Change cycle. */
#define PARM_MODE_PRESET	2	/* Change mode preset. */



/*
 *	Print help.
 */
void PrintUsage()
{
	printf("\
Usage: yiffstat [options]\n\
\n\
    [options] can be any of the following:\n\
\n\
        --recorder <address:port>    Specify which YIFF server to connect to.\n\
\n"
	);


	return;
}



int main(int argc, char *argv[])
{
	int i, status;
	char *strptr = NULL;

	char *con_arg = NULL;

	int parm;
	char val[1024];

	YiffConnection *con = NULL;
	YiffEvent event;



	/* Parse arguments. */
	for(i = 1; i < argc; i++)
	{
	    if(argv[i] == NULL)
		continue;

	    /* Help. */
	    if(strpfx(argv[i], "--h") ||
               strpfx(argv[i], "-h")
	    )
	    {
		PrintUsage();
		return(0);
	    }


	    /* Connect address. */
	    else if(strpfx(argv[i], "--rec") ||
                    strpfx(argv[i], "-rec")
	    )
	    {
		i++;
		if(i < argc)
		{
		    free(con_arg);
		    con_arg = StringCopyAlloc(argv[i]);
		}
		else
		{
		    fprintf(stderr,
			"%s: Requires argument.\n",
			argv[i - 1]
		    );
		}
	    }
	}


	/*
	 *	Connect to YIFF server.
	 */
	con = YIFFConnect(
	    NULL,		/* No start argument. */
	    con_arg
	);
	if(con == NULL)
	{
	    printf("%s: Cannot connect to YIFF server.\n",
		(con_arg == NULL) ? DEF_CON_ARG : con_arg
	    );

            free(con_arg);
            con_arg = NULL;

	    return(-1);
	}



	/*
	 *	Request stats.
	 */
	YIFFRequestStats(con);

	for(i = 0; i < 30; i++)
	{
            YIFFGetNextEvent(
                con,
                &event,
                True
            );
	    if(event.type != YEventSystemStats)
	    {
	        usleep(1000000);
		continue;
	    }

	    /* Print stats. */
	    printf("recorder: %s\n",
		(con_arg == NULL) ? DEF_CON_ARG : con_arg
	    );
            printf("protocol version: %s\n", "not available");
            printf("vendor: %s\n", "Wolfpack");
            printf("maximum protocol statement: %i bytes\n",
		MAX_PROTOCOL_STATEMENT
	    );

            printf("refresh cycle: %i microseconds\n",
		event.sysstats.cycle_us
	    );
            printf("compensated refresh cycle: %i microseconds\n",
                event.sysstats.compensated_cycle_us
            );
            printf("sound buffer fragments: %i\n",
		event.sysstats.num_fragments
	    );
	    printf("sound buffer size: %i bytes\n",
		event.sysstats.buf_len
	    );
            printf("sample rate: %i Hertz\n",
                event.sysstats.sample_rate
            );
            printf("sample size: %i bits\n",
                event.sysstats.sample_size
            );
            printf("channels: %i\n",
                event.sysstats.channels
            );

            printf("cycle load: %.4lf percent\n",
                event.sysstats.cycle_load * 100
            );

	    break;
	}


	/*
	 *	Disconnect from YIFF server
	 */
	YIFFDisconnect(con, False);
	con = NULL;

	free(con_arg);
	con_arg = NULL;


	return(0);
}
